#ifndef ATOL_DTOX_FPTR_H
#define ATOL_DTOX_FPTR_H

#if defined(_WIN32) || defined(_WIN32_WCE)
#  if defined(DTOX_LIBRARY)
#    define DTOX_SHARED_EXPORT __declspec(dllexport)
#  else
#    define DTOX_SHARED_EXPORT __declspec(dllimport)
#  endif
#  define DTOX_SHARED_CCA __cdecl
#elif defined(__linux__) || defined(ANDROID)
#  if __GNUC__ >= 4
#    define DTOX_SHARED_EXPORT __attribute__((visibility("default")))
#  else
#    define DTOX_SHARED_EXPORT
#  endif
#  define DTOX_SHARED_CCA
#elif defined(__APPLE__)
#  define DTOX_SHARED_EXPORT __attribute__((visibility("default")))
#  define DTOX_SHARED_CCA
#else
#  define DTOX_SHARED_EXPORT
#  define DTOX_SHARED_CCA __attribute__ ((cdecl))
#endif

#ifndef uchar
typedef unsigned char uchar;
#endif

#ifndef uint
typedef unsigned int uint;
#endif

typedef void *libfptr_handle;


#ifdef __cplusplus
extern "C"
{
#endif

/*!
   \ingroup libfptr_group_errors

   \brief Список ошибок
 */
enum libfptr_error
{
    //! Ошибок нет.
        LIBFPTR_OK = 0,
    //! Соединение не установлено
        LIBFPTR_ERROR_CONNECTION_DISABLED,
    //! Нет связи.
        LIBFPTR_ERROR_NO_CONNECTION,
    //! Порт занят.
        LIBFPTR_ERROR_PORT_BUSY,
    //! Порт недоступен.
        LIBFPTR_ERROR_PORT_NOT_AVAILABLE,
    //! Некорректные данные от устройства.
        LIBFPTR_ERROR_INCORRECT_DATA,
    //! Внутренняя ошибка библиотеки.
        LIBFPTR_ERROR_INTERNAL,
    //! Неподдерживаемое преобразование типа параметра.
        LIBFPTR_ERROR_UNSUPPORTED_CAST,
    //! Не найден обязательный параметр.
        LIBFPTR_ERROR_NO_REQUIRED_PARAM,
    //! Некорректные настройки.
        LIBFPTR_ERROR_INVALID_SETTINGS,
    //! Драйвер не настроен.
        LIBFPTR_ERROR_NOT_CONFIGURED,
    //! Не поддерживается в данной версии.
        LIBFPTR_ERROR_NOT_SUPPORTED,
    //! Не поддерживается в данном режиме.
        LIBFPTR_ERROR_INVALID_MODE,
    //! Нeкорректное значение параметра.
        LIBFPTR_ERROR_INVALID_PARAM,
    //! Не удалось загрузить библиотеку.
        LIBFPTR_ERROR_NOT_LOADED,
    //! Неизвестная ошибка.
        LIBFPTR_ERROR_UNKNOWN,

    //! Неверная цена (сумма).
        LIBFPTR_ERROR_INVALID_SUM,
    //! Неверное количество.
        LIBFPTR_ERROR_INVALID_QUANTITY,
    //! Переполнение счетчика наличности.
        LIBFPTR_ERROR_CASH_COUNTER_OVERFLOW,
    //! Невозможно сторно последней операции.
        LIBFPTR_ERROR_LAST_OPERATION_STORNO_DENIED,
    //! Сторно по коду невозможно.
        LIBFPTR_ERROR_STORNO_BY_CODE_DENIED,
    //! Невозможен повтор последней операции.
        LIBFPTR_ERROR_LAST_OPERATION_NOT_REPEATABLE,
    //! Повторная скидка на операцию невозможна.
        LIBFPTR_ERROR_DISCOUNT_NOT_REPEATABLE,
    //! Невозможно начислить скидку/надбавку.
        LIBFPTR_ERROR_DISCOUNT_DENIED,
    //! Неверный код товара.
        LIBFPTR_ERROR_INVALID_COMMODITY_CODE,
    //! Неверный штрихкод товара.
        LIBFPTR_ERROR_INVALID_COMMODITY_BARCODE,
    //! Неверный формат команды.
        LIBFPTR_ERROR_INVALID_COMMAND_FORMAT,
    //! Неверная длина.
        LIBFPTR_ERROR_INVALID_COMMAND_LENGTH,
    //! ККТ заблокирована в режиме ввода даты.
        LIBFPTR_ERROR_BLOCKED_IN_DATE_INPUT_MODE,
    //! Требуется подтверждение ввода даты.
        LIBFPTR_ERROR_NEED_DATE_ACCEPT,
    //! Нет больше данных.
        LIBFPTR_ERROR_NO_MORE_DATA,
    //! Нет подтверждения или отмены продажи.
        LIBFPTR_ERROR_NO_ACCEPT_OR_CANCEL,
    //! Отчет с гашением прерван. Вход в режим невозможен.
        LIBFPTR_ERROR_BLOCKED_BY_REPORT_INTERRUPTION,
    //! Отключение контроля наличности невозможно (не настроены необходимые типы оплаты).
        LIBFPTR_ERROR_DISABLE_CASH_CONTROL_DENIED,
    //! Вход в режим заблокирован.
        LIBFPTR_ERROR_MODE_BLOCKED,
    //! Проверьте дату и время.
        LIBFPTR_ERROR_CHECK_DATE_TIME,
    //! Дата и время в ККТ меньше чем в ФН.
        LIBFPTR_ERROR_DATE_TIME_LESS_THAN_FS,
    //! Невозможно закрыть архив.
        LIBFPTR_ERROR_CLOSE_ARCHIVE_DENIED,
    //! Товар не найден.
        LIBFPTR_ERROR_COMMODITY_NOT_FOUND,
    //! Весовой штрихкод с количеством <> 1.000.
        LIBFPTR_ERROR_WEIGHT_BARCODE_WITH_INVALID_QUANTITY,
    //! Переполнение буфера чека.
        LIBFPTR_ERROR_RECEIPT_BUFFER_OVERFLOW,
    //! Недостаточное количество товара.
        LIBFPTR_ERROR_QUANTITY_TOO_FEW,
    //! Сторнируемое количество больше проданного.
        LIBFPTR_ERROR_STORNO_TOO_MUCH,
    //! Товар не найден
        LIBFPTR_ERROR_BLOCKED_COMMODITY_NOT_FOUND,
    //! Нет бумаги.
        LIBFPTR_ERROR_NO_PAPER,
    //! Открыта крышка.
        LIBFPTR_ERROR_COVER_OPENED,
    //! Нет связи с принтером чеков.
        LIBFPTR_ERROR_PRINTER_FAULT,
    //! Механическая ошибка печатающего устройства.
        LIBFPTR_ERROR_MECHANICAL_FAULT,
    //! Неверный тип чека.
        LIBFPTR_ERROR_INVALID_RECEIPT_TYPE,
    //! Недопустимое целевое устройство.
        LIBFPTR_ERROR_INVALID_UNIT_TYPE,
    //! Нет места в массиве картинок/штрихкодов.
        LIBFPTR_ERROR_NO_MEMORY,
    //! Неверный номер картинки/штрихкода (картинка/штрихкод отсутствует).
        LIBFPTR_ERROR_PICTURE_NOT_FOUND,
    //! Сумма не наличных платежей превышает сумму чека.
        LIBFPTR_ERROR_NONCACH_PAYMENTS_TOO_MUCH,
    //! Накопление меньше суммы возврата или аннулирования.
        LIBFPTR_ERROR_RETURN_DENIED,
    //! Переполнение суммы платежей.
        LIBFPTR_ERROR_PAYMENTS_OVERFLOW,
    //! Предыдущая операция незавершена.
        LIBFPTR_ERROR_BUSY,
    //! Ошибка GSM-модуля.
        LIBFPTR_ERROR_GSM,
    //! Неверная величина скидки / надбавки.
        LIBFPTR_ERROR_INVALID_DISCOUNT,
    //! Операция после скидки / надбавки невозможна.
        LIBFPTR_ERROR_OPERATION_AFTER_DISCOUNT_DENIED,
    //! Неверная секция.
        LIBFPTR_ERROR_INVALID_DEPARTMENT,
    //! Неверный вид оплаты.
        LIBFPTR_ERROR_INVALID_PAYMENT_TYPE,
    //! Переполнение при умножении.
        LIBFPTR_ERROR_MULTIPLICATION_OVERFLOW,
    //! Операция запрещена в таблице настроек.
        LIBFPTR_ERROR_DENIED_BY_SETTINGS,
    //! Переполнение итога чека.
        LIBFPTR_ERROR_TOTAL_OVERFLOW,
    //! Открыт чек аннулирования – операция невозможна.
        LIBFPTR_ERROR_DENIED_IN_ANNULATION_RECEIPT,
    //! Переполнение буфера контрольной ленты.
        LIBFPTR_ERROR_JOURNAL_OVERFLOW,
    //! Чек оплачен не полностью.
        LIBFPTR_ERROR_NOT_FULLY_PAID,
    //! Открыт чек возврата – операция невозможна.
        LIBFPTR_ERROR_DENIED_IN_RETURN_RECEIPT,
    //! Смена превысила 24 часа.
        LIBFPTR_ERROR_SHIFT_EXPIRED,
    //! Открыт чек продажи – операция невозможна.
        LIBFPTR_ERROR_DENIED_IN_SELL_RECEIPT,
    //! Переполнение ФП.
        LIBFPTR_ERROR_FISCAL_MEMORY_OVERFLOW,
    //! Неверный пароль.
        LIBFPTR_ERROR_INVALID_PASSWORD,
    //! Идет обработка контрольной ленты.
        LIBFPTR_ERROR_JOURNAL_BUSY,
    //! Смена закрыта - операция невозможна.
        LIBFPTR_ERROR_DENIED_IN_CLOSED_SHIFT,
    //! Неверный номер таблицы.
        LIBFPTR_ERROR_INVALID_TABLE_NUMBER,
    //! Неверный номер ряда.
        LIBFPTR_ERROR_INVALID_ROW_NUMBER,
    //! Неверный номер поля.
        LIBFPTR_ERROR_INVALID_FIELD_NUMBER,
    //! Неверная дата и/или время.
        LIBFPTR_ERROR_INVALID_DATE_TIME,
    //! Неверная сумма сторно
        LIBFPTR_ERROR_INVALID_STORNO_SUM,
    //! Подсчет суммы сдачи невозможен.
        LIBFPTR_ERROR_CHANGE_CALCULATION,
    //! В ККТ нет денег для выплаты.
        LIBFPTR_ERROR_NO_CASH,
    //! Чек закрыт – операция невозможна.
        LIBFPTR_ERROR_DENIED_IN_CLOSED_RECEIPT,
    //! Чек открыт – операция невозможна.
        LIBFPTR_ERROR_DENIED_IN_OPENED_RECEIPT,
    //! Смена открыта, операция невозможна.
        LIBFPTR_ERROR_DENIED_IN_OPENED_SHIFT,
    //! Серийный номер / MAC-адрес уже задан.
        LIBFPTR_ERROR_SERIAL_NUMBER_ALREADY_ENTERED,
    //! Исчерпан лимит перерегистраций.
        LIBFPTR_ERROR_TOO_MUCH_REREGISTRATIONS,
    //! Неверный номер смены.
        LIBFPTR_ERROR_INVALID_SHIFT_NUMBER,
    //! Недопустимый серийный номер ККТ.
        LIBFPTR_ERROR_INVALID_SERIAL_NUMBER,
    //! Недопустимый РНМ и/или ИНН.
        LIBFPTR_ERROR_INVALID_RNM_VATIN,
    //! ККТ не фискализирована.
        LIBFPTR_ERROR_FISCAL_PRINTER_NOT_ACTIVATED,
    //! Не задан серийный номер.
        LIBFPTR_ERROR_SERIAL_NUMBER_NOT_ENTERED,
    //! Нет отчетов.
        LIBFPTR_ERROR_NO_MORE_REPORTS,
    //! Режим не активизирован.
        LIBFPTR_ERROR_MODE_NOT_ACTIVATED,
    //! Нет указанного чека в КЛ.
        LIBFPTR_ERROR_RECORD_NOT_FOUND_IN_JOURNAL,
    //! Некорректный код или номер кода защиты ККТ.
        LIBFPTR_ERROR_INVALID_LICENSE,
    //! Требуется выполнение общего гашения.
        LIBFPTR_ERROR_NEED_FULL_RESET,
    //! Команда не разрешена введенными кодами защиты ККТ.
        LIBFPTR_ERROR_DENIED_BY_LICENSE,
    //! Невозможна отмена скидки/надбавки.
        LIBFPTR_ERROR_DISCOUNT_CANCELLATION_DENIED,
    //! Невозможно закрыть чек данным типом оплаты.
        LIBFPTR_ERROR_CLOSE_RECEIPT_DENIED,
    //! Неверный номер маршрута.
        LIBFPTR_ERROR_INVALID_ROUTE_NUMBER,
    //! Неверный номер начальной зоны.
        LIBFPTR_ERROR_INVALID_START_ZONE_NUMBER,
    //! Неверный номер конечной зоны.
        LIBFPTR_ERROR_INVALID_END_ZONE_NUMBER,
    //! Неверный тип тарифа.
        LIBFPTR_ERROR_INVALID_RATE_TYPE,
    //! Неверный тариф.
        LIBFPTR_ERROR_INVALID_RATE,
    //! Ошибка обмена с фискальным модулем.
        LIBFPTR_ERROR_FISCAL_MODULE_EXCHANGE,
    //! Необходимо провести профилактические работы.
        LIBFPTR_ERROR_NEED_TECHNICAL_SUPPORT,
    //! Неверные номера смен в ККТ и ФН.
        LIBFPTR_ERROR_SHIFT_NUMBERS_DID_NOT_MATCH,
    //! Нет устройства, обрабатывающего данную команду.
        LIBFPTR_ERROR_DEVICE_NOT_FOUND,
    //! Нет связи с внешним устройством.
        LIBFPTR_ERROR_EXTERNAL_DEVICE_CONNECTION,
    //! Ошибочное состояние ТРК.
        LIBFPTR_ERROR_DISPENSER_INVALID_STATE,
    //! Недопустимое кол-во позиций в чеке.
        LIBFPTR_ERROR_INVALID_POSITIONS_COUNT,
    //! Ошибочный номер ТРК.
        LIBFPTR_ERROR_DISPENSER_INVALID_NUMBER,
    //! Неверный делитель.
        LIBFPTR_ERROR_INVALID_DIVIDER,
    //! Активация данного ФН в составе данной ККТ невозможна.
        LIBFPTR_ERROR_FN_ACTIVATION_DENIED,
    //! Перегрев головки принтера.
        LIBFPTR_ERROR_PRINTER_OVERHEAT,
    //! Ошибка обмена с ФН на уровне интерфейса I2C.
        LIBFPTR_ERROR_FN_EXCHANGE,
    //! Ошибка формата передачи ФН.
        LIBFPTR_ERROR_FN_INVALID_FORMAT,
    //! Неверное состояние ФН.
        LIBFPTR_ERROR_FN_INVALID_STATE,
    //! Неисправимая ошибка ФН.
        LIBFPTR_ERROR_FN_FAULT,
    //! Ошибка КС ФН.
        LIBFPTR_ERROR_FN_CRYPTO_FAULT,
    //! Закончен срок эксплуатации ФН.
        LIBFPTR_ERROR_FN_EXPIRED,
    //! Архив ФН переполнен.
        LIBFPTR_ERROR_FN_OVERFLOW,
    //! В ФН переданы неверная дата или время.
        LIBFPTR_ERROR_FN_INVALID_DATE_TIME,
    //! В ФН нет запрошенных данных.
        LIBFPTR_ERROR_FN_NO_MORE_DATA,
    //! Переполнение ФН (итог чека / смены).
        LIBFPTR_ERROR_FN_TOTAL_OVERFLOW,
    //! Буфер переполнен.
        LIBFPTR_ERROR_BUFFER_OVERFLOW,
    //! Невозможно напечатать вторую фискальную копию.
        LIBFPTR_ERROR_PRINT_SECOND_COPY_DENIED,
    //! Требуется гашение ЭЖ.
        LIBFPTR_ERROR_NEED_RESET_JOURNAL,
    //! Сумма налога больше суммы регистраций по чеку и/или итога.
        LIBFPTR_ERROR_TAX_SUM_TOO_MUCH,
    //! Начисление налога на последнюю операцию невозможно.
        LIBFPTR_ERROR_TAX_ON_LAST_OPERATION_DENIED,
    //! Неверный номер ФН.
        LIBFPTR_ERROR_INVALID_FN_NUMBER,
    //! Сумма сторно налога больше суммы зарегистрированного налога данного типа.
        LIBFPTR_ERROR_TAX_CANCEL_DENIED,
    //! Операция невозможна, недостаточно питания.
        LIBFPTR_ERROR_LOW_BATTERY,
    //! Некорректное значение параметров команды ФН.
        LIBFPTR_ERROR_FN_INVALID_COMMAND,
    //! Превышение размеров TLV данных ФН.
        LIBFPTR_ERROR_FN_COMMAND_OVERFLOW,
    //! Нет транспортного соединения ФН.
        LIBFPTR_ERROR_FN_NO_TRANSPORT_CONNECTION,
    //! Исчерпан ресурс КС ФН.
        LIBFPTR_ERROR_FN_CRYPTO_HAS_EXPIRED,
    //! Ресурс хранения ФД исчерпан.
        LIBFPTR_ERROR_FN_RESOURCE_HAS_EXPIRED,
    //! Сообщение от ОФД не может быть принято ФН.
        LIBFPTR_ERROR_INVALID_MESSAGE_FROM_OFD,
    //! В ФН есть неотправленные ФД.
        LIBFPTR_ERROR_FN_HAS_NOT_SEND_DOCUMENTS,
    //! Исчерпан ресурс ожидания передачи сообщения в ФН.
        LIBFPTR_ERROR_FN_TIMEOUT,
    //! Продолжительность смены ФН более 24 часов.
        LIBFPTR_ERROR_FN_SHIFT_EXPIRED,
    //! Неверная разница во времени между двумя операциями ФН.
        LIBFPTR_ERROR_FN_INVALID_TIME_DIFFERENCE,
    //! Некорректная СНО
        LIBFPTR_ERROR_INVALID_TAXATION_TYPE,
    //! Недопустимый номер ставки налога
        LIBFPTR_ERROR_INVALID_TAX_TYPE,
    //! Недопустимый тип оплаты товара
        LIBFPTR_ERROR_INVALID_COMMODITY_PAYMENT_TYPE,
    //! Недопустимый тип кода товара
        LIBFPTR_ERROR_INVALID_COMMODITY_CODE_TYPE,
    //! Недопустима регистрация подакцизного товара
        LIBFPTR_ERROR_EXCISABLE_COMMODITY_DENIED,
    //! Ошибка программирования реквизита
        LIBFPTR_ERROR_FISCAL_PROPERTY_WRITE,
    //! Неверный тип счетчика.
        LIBFPTR_ERROR_INVALID_COUNTER_TYPE,
    //! Ошибка отрезчика.
        LIBFPTR_ERROR_CUTTER_FAULT,
    //! Снятие отчета прервалось.
        LIBFPTR_ERROR_REPORT_INTERRUPTED,
    //! Недопустимое значение отступа слева.
        LIBFPTR_ERROR_INVALID_LEFT_MARGIN,
    //! Недопустимое значение выравнивания.
        LIBFPTR_ERROR_INVALID_ALIGNMENT,
    //! Недопустимое значение режима работы с налогом.
        LIBFPTR_ERROR_INVALID_TAX_MODE,
    //! Файл не найден или неверный формат.
        LIBFPTR_ERROR_FILE_NOT_FOUND,
    //! Размер картинки слишком большой.
        LIBFPTR_ERROR_PICTURE_TOO_BIG,
    //! Не удалось сформировать штрихкод.
        LIBFPTR_ERROR_INVALID_BARCODE_PARAMS,
    //! Неразрешенные реквизиты.
        LIBFPTR_ERROR_FISCAL_PROPERTY_DENIED,
    //! Ошибка интерфейса ФН.
        LIBFPTR_ERROR_FN_INTERFACE,
    //! Дублирование данных.
        LIBFPTR_ERROR_DATA_DUPLICATE,
    //! Не указаны обызательные реквизиты.
        LIBFPTR_ERROR_NO_REQUIRED_FISCAL_PROPERTY,
    //! Ошибка чтения документа из ФН.
        LIBFPTR_ERROR_FN_READ_DOCUMENT,
    //! Переполнение чисел с плавающей точкой.
        LIBFPTR_ERROR_FLOAT_OVERFLOW,
    //! Неверное значение параметра ККТ.
        LIBFPTR_ERROR_INVALID_SETTING_VALUE,
    //! Внутренняя ошибка ККТ.
        LIBFPTR_ERROR_HARD_FAULT,
    //! ФН не найден.
        LIBFPTR_ERROR_FN_NOT_FOUND,
    //! Невозможно записать реквизит агента.
        LIBFPTR_ERROR_INVALID_AGENT_FISCAL_PROPERTY,
    //! Недопустимое сочетания реквизитов 1002 и 1056.
        LIBFPTR_ERROR_INVALID_FISCAL_PROPERTY_VALUE_1002_1056,
    //! Недопустимое сочетания реквизитов 1002 и 1017.
        LIBFPTR_ERROR_INVALID_FISCAL_PROPERTY_VALUE_1002_1017,
    //! Ошибка скриптового движка ККТ.
        LIBFPTR_ERROR_SCRIPT,
    //! Неверный номер пользовательской ячейки памяти.
        LIBFPTR_ERROR_INVALID_USER_MEMORY_INDEX,
    //! Кассир не зарегистрирован.
        LIBFPTR_ERROR_NO_ACTIVE_OPERATOR,
    //! Отчет о регистрации ККТ прерван. Устраните неисправность и повторите операцию.
        LIBFPTR_ERROR_REGISTRATION_REPORT_INTERRUPTED,
    //! Отчет о закрытии ФН прерван. Устраните неисправность и повторите операцию.
        LIBFPTR_ERROR_CLOSE_FN_REPORT_INTERRUPTED,
    //! Отчет об открытии смены прерван. Устраните неисправность и повторите операцию.
        LIBFPTR_ERROR_OPEN_SHIFT_REPORT_INTERRUPTED,
    //! Отчет о состоянии расчетов прерван. Устраните неисправность и повторите операцию.
        LIBFPTR_ERROR_OFD_EXCHANGE_REPORT_INTERRUPTED,
    //! Закрытие чека прервано.
        LIBFPTR_ERROR_CLOSE_RECEIPT_INTERRUPTED,
    //! Получение документа из ФН прервано.
        LIBFPTR_ERROR_FN_QUERY_INTERRUPTED,
    //! Сбой часов.
        LIBFPTR_ERROR_RTC_FAULT,
    //! Сбой памяти.
        LIBFPTR_ERROR_MEMORY_FAULT,
    //! Сбой микросхемы.
        LIBFPTR_ERROR_CHIP_FAULT,
    //! Ошибка шаблонов документов.
        LIBFPTR_ERROR_TEMPLATES_CORRUPTED,
    //! Недопустимое значение MAC-адреса.
        LIBFPTR_ERROR_INVALID_MAC_ADDRESS,
    //! Неверный тип (номер) скрипта
        LIBFPTR_ERROR_INVALID_SCRIPT_NUMBER,
    //! Загруженные скрипты повреждены или отсутствуют
        LIBFPTR_ERROR_SCRIPTS_FAULT,
    //! Несовместимая версия загруженных скриптов
        LIBFPTR_ERROR_INVALID_SCRIPTS_VERSION,
    //! Ошибка в формате клише
        LIBFPTR_ERROR_INVALID_CLICHE_FORMAT,
    //! Требуется перезагрузка ККТ.
        LIBFPTR_ERROR_WAIT_FOR_REBOOT,
    //! Нет подходящей лицензии.
        LIBFPTR_ERROR_NO_LICENSE,
    //! Неверная версия ФФД
        LIBFPTR_ERROR_INVALID_FFD_VERSION,
    //! Параметр доступен только для чтения
        LIBFPTR_ERROR_CHANGE_SETTING_DENIED,
    //! Неверный тип кода товарной номенклатуры
        LIBFPTR_ERROR_INVALID_NOMENCLATURE_TYPE,
    //! Неверное значение GTIN
        LIBFPTR_ERROR_INVALID_GTIN,
    //! Отрицательный результат математической операции
        LIBFPTR_ERROR_NEGATIVE_MATH_RESULT,
    //! Недопустимое сочетание реквизитов
        LIBFPTR_ERROR_FISCAL_PROPERTIES_COMBINATION,

    //! База кодов ошибок JSON и WEB
        LIBFPTR_ERROR_BASE_WEB = 500,
    //! Ошибка парсинга чека
        LIBFPTR_ERROR_RECEIPT_PARSE_ERROR,
};

/*!
   \ingroup libfptr_group_params

   \brief Используемые параметры методов

   Параметры LIBFPTR_PARAM_* являются параметрами libfptr10.
   Значения данных параметров являются параметрами методов интеграционной библиотеки.
   Использование параметров, отсутствующих в списке, приведет к ошибке выполение метода.
   Если метод не использует некоторые из переданных параметров, он их проигнорирует.

   Параметры, не указанные в этом перечислении и имеющие номера меньше 65535, считаются реквизитами фискального накопителя.
   При таких значений как входящих параметров методов они будут пересланы в ФН (либо применены как параметры команды ККТ).
   За подробным описанием реквизитов обращаться в документ "Форматы фискальных документов".
   Каждый метод библиотеки, помимо указанных у него выходных параметров, может вернуть в параметре \ref LIBFPTR_PARAM_ERROR_TAG_NUMBER номер реквизита,
    при записи которого произошла ошибка. Текстовое описание ошибки можно прочитать с помощью \ref libfptr_error_description().
 */
enum libfptr_param
{
    //! Первый номер параметра
        LIBFPTR_PARAM_FIRST = 65536,
    //! Строка текста
        LIBFPTR_PARAM_TEXT = LIBFPTR_PARAM_FIRST,
    //! Перенос текста
        LIBFPTR_PARAM_TEXT_WRAP,
    //! Выравнивание текста, штрихкодов и картинок
        LIBFPTR_PARAM_ALIGNMENT,

    //! Шрифт текста
        LIBFPTR_PARAM_FONT,
    //! Текст двойной ширины
        LIBFPTR_PARAM_FONT_DOUBLE_WIDTH,
    //! Текст двойной высоты
        LIBFPTR_PARAM_FONT_DOUBLE_HEIGHT,
    //! Межстрочный интервал
        LIBFPTR_PARAM_LINESPACING,
    //! Яркость текста
        LIBFPTR_PARAM_BRIGHTNESS,

    //! Код модели ККТ
        LIBFPTR_PARAM_MODEL,
    //! Тип чека
        LIBFPTR_PARAM_RECEIPT_TYPE,
    //! Тип отчета
        LIBFPTR_PARAM_REPORT_TYPE,
    //! Режим
        LIBFPTR_PARAM_MODE,
    //! Тип внешнего устройства
        LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE,
    //! Данные внешнего устройства
        LIBFPTR_PARAM_EXTERNAL_DEVICE_DATA,
    //! Частота звукового сигнала
        LIBFPTR_PARAM_FREQUENCY,
    //! Длительность звукового сигнала
        LIBFPTR_PARAM_DURATION,
    //! Тип отрезки чековой ленты
        LIBFPTR_PARAM_CUT_TYPE,
    //! Время включения денежного ящика
        LIBFPTR_PARAM_DRAWER_ON_TIMEOUT,
    //! Время выключения денежного ящика
        LIBFPTR_PARAM_DRAWER_OFF_TIMEOUT,
    //! Количество импульсов денежного ящика
        LIBFPTR_PARAM_DRAWER_ON_QUANTITY,
    //! Время ожидания ответа ККТ
        LIBFPTR_PARAM_TIMEOUT_ENQ,
    //! Данные команды ККТ
        LIBFPTR_PARAM_COMMAND_BUFFER,
    //! Данные ответа ККТ
        LIBFPTR_PARAM_ANSWER_BUFFER,
    //! Серийный номер
        LIBFPTR_PARAM_SERIAL_NUMBER,
    //! Код производителя
        LIBFPTR_PARAM_MANUFACTURER_CODE,
    //! Не ждать ответа от устройства
        LIBFPTR_PARAM_NO_NEED_ANSWER,
    //! Сумма информационной скидки
        LIBFPTR_PARAM_INFO_DISCOUNT_SUM,
    //! Передавать только ставку налога
        LIBFPTR_PARAM_USE_ONLY_TAX_TYPE,
    //! Тип оплаты
        LIBFPTR_PARAM_PAYMENT_TYPE,
    //! Сумма оплаты
        LIBFPTR_PARAM_PAYMENT_SUM,
    //! Неоплаченный остаток чека
        LIBFPTR_PARAM_REMAINDER,
    //! Сдача
        LIBFPTR_PARAM_CHANGE,
    //! Номер отдела
        LIBFPTR_PARAM_DEPARTMENT,
    //! Тип налога
        LIBFPTR_PARAM_TAX_TYPE,
    //! Сумма налога
        LIBFPTR_PARAM_TAX_SUM,
    //! Способ начисления налога
        LIBFPTR_PARAM_TAX_MODE,
    //! Электронный чек
        LIBFPTR_PARAM_RECEIPT_ELECTRONICALLY,
    //! Пароль пользователя
        LIBFPTR_PARAM_USER_PASSWORD,
    //! Масштаб
        LIBFPTR_PARAM_SCALE,
    //! Отступ слева
        LIBFPTR_PARAM_LEFT_MARGIN,
    //! Данные штрихкода
        LIBFPTR_PARAM_BARCODE,
    //! Тип штрихкода
        LIBFPTR_PARAM_BARCODE_TYPE,
    //! Печатать текст штрихкода
        LIBFPTR_PARAM_BARCODE_PRINT_TEXT,
    //! Версия штрихкода
        LIBFPTR_PARAM_BARCODE_VERSION,
    //! Коррекция штрихкода
        LIBFPTR_PARAM_BARCODE_CORRECTION,
    //! Количество столбцов штрихкода
        LIBFPTR_PARAM_BARCODE_COLUMNS,
    //! Инвертировать штрихкод
        LIBFPTR_PARAM_BARCODE_INVERT,
    //! Высота
        LIBFPTR_PARAM_HEIGHT,
    //! Ширина
        LIBFPTR_PARAM_WIDTH,
    //! Имя файла
        LIBFPTR_PARAM_FILENAME,
    //! Номер картинки
        LIBFPTR_PARAM_PICTURE_NUMBER,
    //! Тип данных ККТ
        LIBFPTR_PARAM_DATA_TYPE,
    //! Номер кассира
        LIBFPTR_PARAM_OPERATOR_ID,
    //! Номер ККТ в магазине
        LIBFPTR_PARAM_LOGICAL_NUMBER,
    //! Дата и время
        LIBFPTR_PARAM_DATE_TIME,
    //! Признак фискальности ККТ
        LIBFPTR_PARAM_FISCAL,
    //! Состояние смены
        LIBFPTR_PARAM_SHIFT_STATE,
    //! Денежный ящик открыт
        LIBFPTR_PARAM_CASHDRAWER_OPENED,
    //! Есть бумага
        LIBFPTR_PARAM_RECEIPT_PAPER_PRESENT,
    //! Крышка открыта
        LIBFPTR_PARAM_COVER_OPENED,
    //! Подрежим ККТ
        LIBFPTR_PARAM_SUBMODE,
    //! Номер чека в ККТ
        LIBFPTR_PARAM_RECEIPT_NUMBER,
    //! Номер документа в ККТ
        LIBFPTR_PARAM_DOCUMENT_NUMBER,
    //! Номер смены в ККТ
        LIBFPTR_PARAM_SHIFT_NUMBER,
    //! Сумма чека
        LIBFPTR_PARAM_RECEIPT_SUM,
    //! Ширина чековой ленты в символах
        LIBFPTR_PARAM_RECEIPT_LINE_LENGTH,
    //! Ширина чековой ленты в пикселях
        LIBFPTR_PARAM_RECEIPT_LINE_LENGTH_PIX,
    //! Название модели ККТ
        LIBFPTR_PARAM_MODEL_NAME,
    //! Версия ПО ФР
        LIBFPTR_PARAM_UNIT_VERSION,
    //! Флаг ошибки соединения с печатным механизмом
        LIBFPTR_PARAM_PRINTER_CONNECTION_LOST,
    //! Флаг невосстановимой ошибки принтера
        LIBFPTR_PARAM_PRINTER_ERROR,
    //! Флаг ошибки отрезчика
        LIBFPTR_PARAM_CUT_ERROR,
    //! Флаг перегрева печатающей головки
        LIBFPTR_PARAM_PRINTER_OVERHEAT,
    //! Номер модуля
        LIBFPTR_PARAM_UNIT_TYPE,
    //! Номер кода защиты
        LIBFPTR_PARAM_LICENSE_NUMBER,
    //! Состояние кода защиты
        LIBFPTR_PARAM_LICENSE_ENTERED,
    //! Значение кода защиты
        LIBFPTR_PARAM_LICENSE,
    //! Сумма
        LIBFPTR_PARAM_SUM,
    //! Количество
        LIBFPTR_PARAM_COUNT,
    //! Тип счетчика
        LIBFPTR_PARAM_COUNTER_TYPE,
    //! Тип счетчика шагов
        LIBFPTR_PARAM_STEP_COUNTER_TYPE,
    //! Номер реквизита, на операции с которым произошла ошибка
        LIBFPTR_PARAM_ERROR_TAG_NUMBER,
    //! Номер таблицы
        LIBFPTR_PARAM_TABLE,
    //! Номер строки таблицы
        LIBFPTR_PARAM_ROW,
    //! Номер поля в строке таблицы
        LIBFPTR_PARAM_FIELD,
    //! Значение поля таблицы
        LIBFPTR_PARAM_FIELD_VALUE,
    //! Тип данных ФН
        LIBFPTR_PARAM_FN_DATA_TYPE,
    //! Номер реквизита ФН
        LIBFPTR_PARAM_TAG_NUMBER,
    //! Данные реквизита ФН
        LIBFPTR_PARAM_TAG_VALUE,
    //! Количество документов
        LIBFPTR_PARAM_DOCUMENTS_COUNT,
    //! Фискальный признак документа
        LIBFPTR_PARAM_FISCAL_SIGN,
    //! Номер версии ФФД ККТ
        LIBFPTR_PARAM_DEVICE_FFD_VERSION,
    //! Номер версии ФФД ФН
        LIBFPTR_PARAM_FN_FFD_VERSION,
    //! Номер версии ФФД
        LIBFPTR_PARAM_FFD_VERSION,
    //! Проверять сумму
        LIBFPTR_PARAM_CHECK_SUM,
    //! Наименование товара
        LIBFPTR_PARAM_COMMODITY_NAME,
    //! Цена за единицу товара
        LIBFPTR_PARAM_PRICE,
    //! Количество товара
        LIBFPTR_PARAM_QUANTITY,
    //! Общая сумма позиции
        LIBFPTR_PARAM_POSITION_SUM,
    //! Тип ФН
        LIBFPTR_PARAM_FN_TYPE,
    //! Версия ФН
        LIBFPTR_PARAM_FN_VERSION,
    //! Количество оставшихся перерегистраций
        LIBFPTR_PARAM_REGISTRATIONS_REMAIN,
    //! Количество проведенных регистраций
        LIBFPTR_PARAM_REGISTRATIONS_COUNT,
    //! Не возвращать ошибку если не поддерживается
        LIBFPTR_PARAM_NO_ERROR_IF_NOT_SUPPORTED,
    //! Состояние обмена с ОФД
        LIBFPTR_PARAM_OFD_EXCHANGE_STATUS,
    //! Данные ошибки ФН
        LIBFPTR_PARAM_FN_ERROR_DATA,
    //! Код ошибки ФН
        LIBFPTR_PARAM_FN_ERROR_CODE,
    //! Режим ЕНВД
        LIBFPTR_PARAM_ENVD_MODE,
    //! Состояние документа (закрыт или открыт)
        LIBFPTR_PARAM_DOCUMENT_CLOSED,
    //! JSON-строка
        LIBFPTR_PARAM_JSON_DATA,
    //! Данные команды ККТ
        LIBFPTR_PARAM_COMMAND_SUBSYSTEM,
    //! Тип операции ФН
        LIBFPTR_PARAM_FN_OPERATION_TYPE,
    //! Состояние ФН
        LIBFPTR_PARAM_FN_STATE,
    //! Режим ЕНВД разрешен
        LIBFPTR_PARAM_ENVD_MODE_ENABLED,
    //! Номер параметра ККТ
        LIBFPTR_PARAM_SETTING_ID,
    //! Значение параметра ККТ
        LIBFPTR_PARAM_SETTING_VALUE,
    //! Название настройки
        LIBFPTR_PARAM_MAPPING_KEY,
    //! Список возможных значений настройки
        LIBFPTR_PARAM_MAPPING_VALUE,
    //! Штучный товар
        LIBFPTR_PARAM_COMMODITY_PIECE,
    //! Тип источника питания
        LIBFPTR_PARAM_POWER_SOURCE_TYPE,
    //! Заряд аккумулятора
        LIBFPTR_PARAM_BATTERY_CHARGE,
    //! Напряжение источника питания
        LIBFPTR_PARAM_VOLTAGE,
    //! Устройство работает от аккумулятора
        LIBFPTR_PARAM_USE_BATTERY,
    //! Напряжение источника питания
        LIBFPTR_PARAM_BATTERY_CHARGING,
    //! Возможность печати при текущем заряде аккумулятора
        LIBFPTR_PARAM_CAN_PRINT_WHILE_ON_BATTERY,
    //! MAC-адрес
        LIBFPTR_PARAM_MAC_ADDRESS,
    //! Флаг фискальности ФН
        LIBFPTR_PARAM_FN_FISCAL,
    //! Код ошибки сети
        LIBFPTR_PARAM_NETWORK_ERROR,
    //! Код ошибки ОФД
        LIBFPTR_PARAM_OFD_ERROR,
    //! Код ошибки ФН
        LIBFPTR_PARAM_FN_ERROR,
    //! Код команды
        LIBFPTR_PARAM_COMMAND_CODE,
    //! Температура ТПГ
        LIBFPTR_PARAM_PRINTER_TEMPERATURE,
    //! Тип набора записей
        LIBFPTR_PARAM_RECORDS_TYPE,
    //! Фискальный признак ОФД
        LIBFPTR_PARAM_OFD_FISCAL_SIGN,
    //! Флаг подтверждения ОФД
        LIBFPTR_PARAM_HAS_OFD_TICKET,
    //! Флаг отсутствия ЗН ККТ
        LIBFPTR_PARAM_NO_SERIAL_NUMBER,
    //! Флаг ошибки часов реального времени
        LIBFPTR_PARAM_RTC_FAULT,
    //! Флаг ошибки таблиц настроек
        LIBFPTR_PARAM_SETTINGS_FAULT,
    //! Флаг ошибки счетчиков и регистров
        LIBFPTR_PARAM_COUNTERS_FAULT,
    //! Флаг ошибки пользовательской памяти
        LIBFPTR_PARAM_USER_MEMORY_FAULT,
    //! Флаг ошибки сервисных регистров
        LIBFPTR_PARAM_SERVICE_COUNTERS_FAULT,
    //! Флаг ошибки реквизитов
        LIBFPTR_PARAM_ATTRIBUTES_FAULT,
    //! Флаг фатальной ошибки ФН
        LIBFPTR_PARAM_FN_FAULT,
    //! Флаг неверного ФН
        LIBFPTR_PARAM_INVALID_FN,
    //! Флаг фатальной ошибки оборудования
        LIBFPTR_PARAM_HARD_FAULT,
    //! Флаг ошибки менеджера памяти
        LIBFPTR_PARAM_MEMORY_MANAGER_FAULT,
    //! Флаг ошибки скриптов
        LIBFPTR_PARAM_SCRIPTS_FAULT,
    //! Флаг выполнения полной очистки памяти
        LIBFPTR_PARAM_FULL_RESET,
    //! Флаг требования перезагрузки
        LIBFPTR_PARAM_WAIT_FOR_REBOOT,
    //! Масштаб в процентах
        LIBFPTR_PARAM_SCALE_PERCENT,
    //! Требуется срочная замена ФН
        LIBFPTR_PARAM_FN_NEED_REPLACEMENT,
    //! Исчерпан ресурс ФН
        LIBFPTR_PARAM_FN_RESOURCE_EXHAUSTED,
    //! Память ФН переполнена
        LIBFPTR_PARAM_FN_MEMORY_OVERFLOW,
    //! Превышено время ожидания ответа от ОФД
        LIBFPTR_PARAM_FN_OFD_TIMEOUT,
    //! Критическая ошибка ФН
        LIBFPTR_PARAM_FN_CRITICAL_ERROR,
    //! Флаг чтения сообщения для ОФД
        LIBFPTR_PARAM_OFD_MESSAGE_READ,
    //! Минимальная версия ФФД ККТ
        LIBFPTR_PARAM_DEVICE_MIN_FFD_VERSION,
    //! Максимальная версия ФФД ККТ
        LIBFPTR_PARAM_DEVICE_MAX_FFD_VERSION,
    //! Время работы ККТ
        LIBFPTR_PARAM_DEVICE_UPTIME,
    //! Тип кода товарной номенклатуры
        LIBFPTR_PARAM_NOMENCLATURE_TYPE,
    //! Идентификатор продукта GTIN
        LIBFPTR_PARAM_GTIN,
    //! Тип документа из ФН
        LIBFPTR_PARAM_FN_DOCUMENT_TYPE,
    //! Описание ошибки сети
        LIBFPTR_PARAM_NETWORK_ERROR_TEXT,
    //! Описание ошибки ФН
        LIBFPTR_PARAM_FN_ERROR_TEXT,
    //! Описание ошибки ОФД
        LIBFPTR_PARAM_OFD_ERROR_TEXT,
    //! Идентификатор пользовательского скрипта
        LIBFPTR_PARAM_USER_SCRIPT_ID,
    //! Параметр пользовательского скрипта
        LIBFPTR_PARAM_USER_SCRIPT_PARAMETER,
    //! Тип операции над пользовательской памятью ККТ
        LIBFPTR_PARAM_USER_MEMORY_OPERATION,
    //! Данные пользовательской памяти
        LIBFPTR_PARAM_USER_MEMORY_DATA,
    //! Строка пользовательской памяти
        LIBFPTR_PARAM_USER_MEMORY_STRING,
    //! Адрес в пользовательской памяти
        LIBFPTR_PARAM_USER_MEMORY_ADDRESS,
    //! Флаг наличия фискального накопителя
        LIBFPTR_PARAM_FN_PRESENT,
    //! Флаг блокировки ККТ
        LIBFPTR_PARAM_BLOCKED,
    //! Флаг напечатанного документа
        LIBFPTR_PARAM_DOCUMENT_PRINTED,
    //! Сумма скидок за смену
        LIBFPTR_PARAM_DISCOUNT_SUM,
    //! Сумма надбавок за смену
        LIBFPTR_PARAM_SURCHARGE_SUM,

    //! Последний номер параметра
        LIBFPTR_PARAM_LAST
};

/*!
   \ingroup libfptr_group_lib

   \brief Модель ККТ

   Список возможных значений для параметра \ref LIBFPTR_PARAM_MODEL
 */
enum libfptr_model
{
    //! Неизвестна
        LIBFPTR_MODEL_UNKNOWN = 0,
    //! Автоматически (АТОЛ)
        LIBFPTR_MODEL_ATOL_AUTO = 500,
    //! АТОЛ 11Ф
        LIBFPTR_MODEL_ATOL_11F = 67,
    //! АТОЛ 15Ф
        LIBFPTR_MODEL_ATOL_15F = 78,
    //! АТОЛ 20Ф
        LIBFPTR_MODEL_ATOL_20F = 81,
    //! АТОЛ 22Ф (АТОЛ FPrint-22ПТК)
        LIBFPTR_MODEL_ATOL_22F = 63,
    //! АТОЛ 25Ф
        LIBFPTR_MODEL_ATOL_25F = 57,
    //! АТОЛ 30Ф
        LIBFPTR_MODEL_ATOL_30F = 61,
    //! АТОЛ 42ФС
        LIBFPTR_MODEL_ATOL_42FS = 77,
    //! АТОЛ 50Ф
        LIBFPTR_MODEL_ATOL_50F = 80,
    //! АТОЛ 52Ф
        LIBFPTR_MODEL_ATOL_52F = 64,
    //! АТОЛ 55Ф
        LIBFPTR_MODEL_ATOL_55F = 62,
    //! АТОЛ 60Ф
        LIBFPTR_MODEL_ATOL_60F = 75,
    //! АТОЛ 77Ф
        LIBFPTR_MODEL_ATOL_77F = 69,
    //! АТОЛ 90Ф
        LIBFPTR_MODEL_ATOL_90F = 72,
    //! АТОЛ 91Ф
        LIBFPTR_MODEL_ATOL_91F = 82,
    //! АТОЛ 92Ф
        LIBFPTR_MODEL_ATOL_92F = 84,
    //! АТОЛ Sigma 10 (АТОЛ 150Ф)
        LIBFPTR_MODEL_ATOL_SIGMA_10 = 86
};

/*!
   \ingroup libfptr_group_lib
   \brief Путь к зависимостям библиотеки.
 */
#define LIBFPTR_SETTING_LIBRARY_PATH L"LibraryPath"
/*!
   \ingroup libfptr_group_lib
   \brief Модель ККТ.
 */
#define LIBFPTR_SETTING_MODEL L"Model"
/*!
   \ingroup libfptr_group_lib
   \brief Идентификатор канала обмена с ККТ.
 */
#define LIBFPTR_SETTING_PORT L"Port"
/*!
   \ingroup libfptr_group_lib
   \brief Скорость канала RS232.
 */
#define LIBFPTR_SETTING_BAUDRATE L"BaudRate"
/*!
   \ingroup libfptr_group_lib
   \brief Битность канала RS232.
 */
#define LIBFPTR_SETTING_BITS L"Bits"
/*!
   \ingroup libfptr_group_lib
   \brief Четность канала RS232.
 */
#define LIBFPTR_SETTING_PARITY L"Parity"
/*!
   \ingroup libfptr_group_lib
   \brief Количество стоп-битов канала RS232.
 */
#define LIBFPTR_SETTING_STOPBITS L"StopBits"
/*!
   \ingroup libfptr_group_lib
   \brief IP-адрес ККТ.
 */
#define LIBFPTR_SETTING_IPADDRESS L"IPAddress"
/*!
   \ingroup libfptr_group_lib
   \brief IP-порт ККТ.
 */
#define LIBFPTR_SETTING_IPPORT L"IPPort"
/*!
   \ingroup libfptr_group_lib
   \brief MAC-адрес ККТ канала Bluetooth.
 */
#define LIBFPTR_SETTING_MACADDRESS L"MACAddress"
/*!
   \ingroup libfptr_group_lib
   \brief Название порта канала RS232.
 */
#define LIBFPTR_SETTING_COM_FILE L"ComFile"
/*!
   \ingroup libfptr_group_lib
   \brief Путь к устройсву для канала USB (Linux).
 */
#define LIBFPTR_SETTING_USB_DEVICE_PATH L"UsbDevicePath"
/*!
   \ingroup libfptr_group_lib
   \brief Флаг автоматического включения адаптера Bluetooth при подключении.
 */
#define LIBFPTR_SETTING_BT_AUTOENABLE L"AutoEnableBluetooth"
/*!
   \ingroup libfptr_group_lib
   \brief Флаг автоматического отключения адаптера Bluetooth при отключении.
 */
#define LIBFPTR_SETTING_BT_AUTODISABLE L"AutoDisableBluetooth"
/*!
   \ingroup libfptr_group_lib
   \brief Пароль доступа к ККТ.
 */
#define LIBFPTR_SETTING_ACCESS_PASSWORD L"AccessPassword"
/*!
   \ingroup libfptr_group_lib
   \brief Пароль пользователя ККТ по умолчанию.
 */
#define LIBFPTR_SETTING_USER_PASSWORD L"UserPassword"
/*!
   \ingroup libfptr_group_lib
   \brief Настройка ОФД-канала библиотеки.
 */
#define LIBFPTR_SETTING_OFD_CHANNEL L"OfdChannel"
/*!
   \ingroup libfptr_group_lib
   \brief Список существующих в системе COM-портов. Константа для метода \ref libfptr_util_mapping().
 */
#define LIBFPTR_SETTING_EXISTED_COM_FILES L"ExistedComFiles"

/*!
   \ingroup libfptr_group_lib

   \brief Способ соединения с ККТ
 */
enum libfptr_port
{
    //! RS232 / Virtual COM
        LIBFPTR_PORT_COM = 0,
    //! USB
        LIBFPTR_PORT_USB,
    //! Ethernet и Wi-Fi
        LIBFPTR_PORT_TCPIP,
    //! Bluetooth
        LIBFPTR_PORT_BLUETOOTH,
};

/*!
   \ingroup libfptr_group_lib

   \brief Скорость соединения (RS232)
 */
enum libfptr_baudrate
{
    //! 1200 бод
        LIBFPTR_PORT_BR_1200 = 1200,
    //! 2400 бод
        LIBFPTR_PORT_BR_2400 = 2400,
    //! 4800 бод
        LIBFPTR_PORT_BR_4800 = 4800,
    //! 9600 бод
        LIBFPTR_PORT_BR_9600 = 9600,
    //! 19200 бод
        LIBFPTR_PORT_BR_19200 = 19200,
    //! 38400 бод
        LIBFPTR_PORT_BR_38400 = 38400,
    //! 57600 бод
        LIBFPTR_PORT_BR_57600 = 57600,
    //! 115200 бод
        LIBFPTR_PORT_BR_115200 = 115200,
};

/*!
   \ingroup libfptr_group_lib

   \brief Количество бит в байте (RS232)
 */
enum libfptr_bits
{
    //! 7 бит
        LIBFPTR_PORT_BITS_7 = 7,
    //! 8 бит
        LIBFPTR_PORT_BITS_8 = 8,
};

/*!
   \ingroup libfptr_group_lib

   \brief Четность (RS232)
 */
enum libfptr_parity
{
    //! Нет четности
        LIBFPTR_PORT_PARITY_NO = 0,
    //! Четность
        LIBFPTR_PORT_PARITY_ODD,
    //! Нечетность
        LIBFPTR_PORT_PARITY_EVEN,
    //! Установлен
        LIBFPTR_PORT_PARITY_MARK,
    //! Сброшен
        LIBFPTR_PORT_PARITY_SPACE,
};

/*!
   \ingroup libfptr_group_lib

   \brief Количество стоп-битов (RS232)
 */
enum libfptr_stopbits
{
    //! 1 стоп-бит
        LIBFPTR_PORT_SB_1 = 0,
    //! 1.5 стоп-битов
        LIBFPTR_PORT_SB_1_5,
    //! 2 стоп-бита
        LIBFPTR_PORT_SB_2
};

/*!
   \ingroup libfptr_group_print

   \brief Тип штрихкода

   Список возможных значений для параметра \ref LIBFPTR_PARAM_BARCODE_TYPE
 */
enum libfptr_barcode_type
{
    //! EAN-8
        LIBFPTR_BT_EAN_8 = 0,
    //! EAN-13
        LIBFPTR_BT_EAN_13,
    //! UPC-A
        LIBFPTR_BT_UPC_A,
    //! UPC-E
        LIBFPTR_BT_UPC_E,
    //! Code 39
        LIBFPTR_BT_CODE_39,
    //! Code 93
        LIBFPTR_BT_CODE_93,
    //! Code 128
        LIBFPTR_BT_CODE_128,
    //! Codabar
        LIBFPTR_BT_CODABAR,
    //! Interleaved 2of5
        LIBFPTR_BT_ITF,
    //! ITF-14
        LIBFPTR_BT_ITF_14,
    //! GS1-128 (EAN-128)
        LIBFPTR_BT_GS1_128,
    //! QR-код
        LIBFPTR_BT_QR,
    //! PDF417
        LIBFPTR_BT_PDF417,
    //! AZTEC
        LIBFPTR_BT_AZTEC,
};

/*!
   \ingroup libfptr_group_print

   \brief Уровень коррекции QR-кода и PDF417

   Список возможных значений для параметра \ref LIBFPTR_PARAM_BARCODE_CORRECTION
 */
enum libfptr_barcode_correction
{
    LIBFPTR_BC_DEFAULT = 0,
    //! Минимальная
        LIBFPTR_BC_0,
    LIBFPTR_BC_1,
    LIBFPTR_BC_2,
    //! Максимальная для QR/Aztec
        LIBFPTR_BC_3,
    LIBFPTR_BC_4,
    LIBFPTR_BC_5,
    LIBFPTR_BC_6,
    LIBFPTR_BC_7,
    //! Максимальная для PDF417
        LIBFPTR_BC_8,
};

/*!
   \ingroup libfptr_group_receipt

   \brief Способ начисления налога

   Список возможных значений для параметра \ref LIBFPTR_PARAM_TAX_MODE
 */
enum libfptr_tax_mode
{
    //! На позицию
        LIBFPTR_TM_POSITION = 0,
    //! На единицу
        LIBFPTR_TM_UNIT,
};

/*!
   \ingroup libfptr_group_ecr_query

   \brief Тип счетчика шагов

   Список возможных значений для параметра \ref LIBFPTR_PARAM_STEP_COUNTER_TYPE
 */
enum libfptr_step_counter_type
{
    //! Всего
        LIBFPTR_SCT_OVERALL = 0,
    //! Только вперед
        LIBFPTR_SCT_FORWARD,
};

/*!
   \ingroup libfptr_group_ecr_query

   \brief Тип счетчика

   Список возможных значений для параметра \ref LIBFPTR_PARAM_COUNTER_TYPE
 */
enum libfptr_counter_type
{
    //! Постоянный
        LIBFPTR_CT_ROLLUP = 0,
    //! Сбрасываем при техобнулении
        LIBFPTR_CT_RESETTABLE,
};

/*!
   \ingroup libfptr_group_ecr_query

   \brief Состояние смены

   Список возможных значений для параметра \ref LIBFPTR_PARAM_SHIFT_STATE
 */
enum libfptr_shift_state
{
    //! Смены закрыта
        LIBFPTR_SS_CLOSED = 0,
    //! Смены открыта
        LIBFPTR_SS_OPENED,
    //! Смена превысила 24 часа
        LIBFPTR_SS_EXPIRED,
};

/*!
   \ingroup libfptr_group_misc

   \brief Способ отрезки чековой ленты

   Список возможных значений для параметра \ref LIBFPTR_PARAM_CUT_TYPE
 */
enum libfptr_cut_type
{
    //! Полная отрезка
        LIBFPTR_CT_FULL = 0,
    //! Частичная отрезка
        LIBFPTR_CT_PART,
};

/*!
   \ingroup libfptr_group_text

   \brief Выравнивание текста/графики

   Список возможных значений для параметра \ref LIBFPTR_PARAM_ALIGNMENT
 */
enum libfptr_alignment
{
    //! По левому краю
        LIBFPTR_ALIGNMENT_LEFT = 0,
    //! По центру
        LIBFPTR_ALIGNMENT_CENTER,
    //! По правому краю
        LIBFPTR_ALIGNMENT_RIGHT,
};

/*!
   \ingroup libfptr_group_text

   \brief Перенос текста

   Список возможных значений для параметра \ref LIBFPTR_PARAM_TEXT_WRAP
 */
enum libfptr_text_wrap
{
    //! Не переносить (обрезать по ширине чековой ленты)
        LIBFPTR_TW_NONE = 0,
    //! По словам (разделители слов -  пробел, '.', ',', ';', '-', '?', '!', ')', '}', ']')
        LIBFPTR_TW_WORDS,
    //! По символам (переносить каждые N символов, где N - ширина чековой ленты)
        LIBFPTR_TW_CHARS,
};

/*!
   \ingroup libfptr_group_fn

   \brief Тип фискального накопителя

   Список возможных значений для параметра \ref LIBFPTR_PARAM_FN_TYPE
 */
enum libfptr_fn_type
{
    //! Отладочная версия
        LIBFPTR_FNT_DEBUG = 0,
    //! Боевая версия
        LIBFPTR_FNT_RELEASE,
    //! Неизвестная версия
        LIBFPTR_FNT_UNKNOWN,
};

/*!
   \ingroup libfptr_group_fn

   \brief Состояние фискального накопителя

   Список возможных значений для параметра \ref LIBFPTR_PARAM_FN_STATE
 */
enum libfptr_fn_state
{
    //! Настройка ФН
        LIBFPTR_FNS_INITIAL = 0,
    //! Готовность к активации
        LIBFPTR_FNS_CONFIGURED = 1,
    //! Фискальный режим
        LIBFPTR_FNS_FISCAL_MODE = 3,
    //! Постфискальный режим
        LIBFPTR_FNS_POSTFISCAL_MODE = 7,
    //! Доступ к архиву ФН
        LIBFPTR_FNS_ACCESS_ARCHIVE = 15,
};

/*!
   \ingroup libfptr_group_receipt

   \brief Тип чека

   Список возможных значений для параметра \ref LIBFPTR_PARAM_RECEIPT_TYPE
 */
enum libfptr_receipt_type
{
    //! Чек закрыт
        LIBFPTR_RT_CLOSED = 0,
    //! Чек прихода
        LIBFPTR_RT_SELL = 1,
    //! Чек возврата прихода
        LIBFPTR_RT_SELL_RETURN = 2,
    //! Чек коррекции прихода
        LIBFPTR_RT_SELL_CORRECTION = 7,
    //! Чек расхода
        LIBFPTR_RT_BUY = 4,
    //! Чек возврата расхода
        LIBFPTR_RT_BUY_RETURN = 5,
    //! Чек коррекции расхода
        LIBFPTR_RT_BUY_CORRECTION = 9,
};

/*!
   \ingroup libfptr_group_report

   \brief Тип отчета

   Список возможных значений для параметра \ref LIBFPTR_PARAM_REPORT_TYPE
 */
enum libfptr_report_type
{
    //! Отчет о закрытии смены
        LIBFPTR_RT_CLOSE_SHIFT = 0,
    //! X-отчет
        LIBFPTR_RT_X,
    //! Печать копии последнего документа
        LIBFPTR_RT_LAST_DOCUMENT,
    //! Отчет о состоянии расчетов
        LIBFPTR_RT_OFD_EXCHANGE_STATUS,
    //! Демонстрационная печать ККТ
        LIBFPTR_RT_KKT_DEMO,
    //! Печать информации о ККТ
        LIBFPTR_RT_KKT_INFO,
    //! Тест связи с ОФД
        LIBFPTR_RT_OFD_TEST,
    //! Печать документа по номеру из ФН
        LIBFPTR_RT_FN_DOC_BY_NUMBER,
    //! Отчет количеств
        LIBFPTR_RT_QUANTITY,
    //! Отчет по секциям
        LIBFPTR_RT_DEPARTMENTS,
    //! Отчет по кассирам
        LIBFPTR_RT_OPERATORS,
    //! Почасовой отчет
        LIBFPTR_RT_HOURS,
    //! Печать итогов регистрации/перерегистрации ККТ
        LIBFPTR_RT_FN_REGISTRATIONS,
    //! Печать счетчиков итогов смены
        LIBFPTR_RT_FN_SHIFT_TOTAL_COUNTERS,
    //! Печать счетчиков итогов ФН
        LIBFPTR_RT_FN_TOTAL_COUNTERS,
    //! Печать счетчиков по непереданным документам
        LIBFPTR_RT_FN_NOT_SENT_DOCUMENTS_COUNTERS,
    //! Печать отчета по товарам по СНО
        LIBFPTR_RT_COMMODITIES_BY_TAXATION_TYPES,
    //! Печать отчета по товарам по секциям
        LIBFPTR_RT_COMMODITIES_BY_DEPARTMENTS,
    //! Печать отчета по товарам по суммам
        LIBFPTR_RT_COMMODITIES_BY_SUMS,
    //! Начать служебный отчет
        LIBFPTR_RT_START_SERVICE
};

/*!
   \ingroup libfptr_group_receipt

   \brief Тип оплаты

   Список возможных значений для параметра \ref LIBFPTR_PARAM_PAYMENT_TYPE

   \warning Для использования этих констант требуется, чтобы в ККТ был настроен аналогичный порядок типов оплат.
 */
enum libfptr_payment_type
{
    //! Оплата наличными
        LIBFPTR_PT_CASH = 0,
    //! Электронными
        LIBFPTR_PT_ELECTRONICALLY,
    //! Предварительная оплата (аванс)
        LIBFPTR_PT_PREPAID,
    //! Последующая оплата (кредит)
        LIBFPTR_PT_CREDIT,
    //! Иная форма оплаты (встречное предоставление)
        LIBFPTR_PT_OTHER,
    //! Тип оплаты 6
        LIBFPTR_PT_6,
    //! Тип оплаты 7
        LIBFPTR_PT_7,
    //! Тип оплаты 8
        LIBFPTR_PT_8,
    //! Тип оплаты 9
        LIBFPTR_PT_9,
    //! Тип оплаты 10
        LIBFPTR_PT_10
};

/*!
   \ingroup libfptr_group_receipt

   \brief Налоговая ставка

   Список возможных значений для параметра \ref LIBFPTR_PARAM_TAX_TYPE
 */
enum libfptr_tax_type
{
    //! По секции
        LIBFPTR_TAX_DEPARTMENT = 0,
    //! НДС 18\%
        LIBFPTR_TAX_VAT18 = 1,
    //! НДС 10\%
        LIBFPTR_TAX_VAT10,
    //! НДС расч. 18/118
        LIBFPTR_TAX_VAT118,
    //! НДС расч. 10/110
        LIBFPTR_TAX_VAT110,
    //! НДС 0\%
        LIBFPTR_TAX_VAT0,
    //! Не облагается
        LIBFPTR_TAX_NO,
};

/*!
   \ingroup libfptr_group_external_dev

   \brief Тип внешнего устройства

   Список возможных значений для параметра \ref LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE
 */
enum libfptr_external_device_type
{
    //! Дисплей покупателя
        LIBFPTR_EXTERNAL_DEVICE_DISPLAY = 0,
    //! Платежный терминал
        LIBFPTR_EXTERNAL_DEVICE_PINPAD,
    //! Модем
        LIBFPTR_EXTERNAL_DEVICE_MODEM,
    //! Сканер ШК
        LIBFPTR_EXTERNAL_DEVICE_BARCODE_SCANNER,
};

/*!
   \ingroup libfptr_group_ecr_query

   \brief Тип данных ККТ

   Список возможных значений для параметра \ref LIBFPTR_PARAM_DATA_TYPE
 */
enum libfptr_kkt_data_type
{
    //! Общая информация и статус ККТ
        LIBFPTR_DT_STATUS = 0,
    //! Сумма наличных в денежном ящике
        LIBFPTR_DT_CASH_SUM,
    //! Версия модуля
        LIBFPTR_DT_UNIT_VERSION,
    //! Параметры картинки в памяти / внешнем хранилище
        LIBFPTR_DT_PICTURE_INFO,
    //! Состояние кода защиты
        LIBFPTR_DT_LICENSE_ACTIVATED,
    //! Сумма регистраций
        LIBFPTR_DT_REGISTRATIONS_SUM,
    //! Количество регистраций
        LIBFPTR_DT_REGISTRATIONS_COUNT,
    //! Сумма платежей
        LIBFPTR_DT_PAYMENT_SUM,
    //! Сумма внесений
        LIBFPTR_DT_CASHIN_SUM,
    //! Количество внесений
        LIBFPTR_DT_CASHIN_COUNT,
    //! Сумма выплат
        LIBFPTR_DT_CASHOUT_SUM,
    //! Количество выплат
        LIBFPTR_DT_CASHOUT_COUNT,
    //! Выручка
        LIBFPTR_DT_REVENUE,
    //! Текущие дата и время в ККТ
        LIBFPTR_DT_DATE_TIME,
    //! Состояние смены
        LIBFPTR_DT_SHIFT_STATE,
    //! Состояние чека
        LIBFPTR_DT_RECEIPT_STATE,
    //! Серийный номер
        LIBFPTR_DT_SERIAL_NUMBER,
    //! Информация о ККТ
        LIBFPTR_DT_MODEL_INFO,
    //! Ширина ленты
        LIBFPTR_DT_RECEIPT_LINE_LENGTH,
    //! Ресурс отрезчика
        LIBFPTR_DT_CUTTER_RESOURCE,
    //! Ресурс ШД
        LIBFPTR_DT_STEP_RESOURCE,
    //! Ресурс ТПГ
        LIBFPTR_DT_TERMAL_RESOURCE,
    //! Режим ЕНВД
        LIBFPTR_DT_ENVD_MODE,
    //! Сумма налога за смену
        LIBFPTR_DT_SHIFT_TAX_SUM,
    //! Сумма налога на чек
        LIBFPTR_DT_RECEIPT_TAX_SUM,
    //! Необнуляемая сумма
        LIBFPTR_DT_NON_NULLABLE_SUM,
    //! Количество чеков
        LIBFPTR_DT_RECEIPT_COUNT,
    //! Количество аннулирований всего
        LIBFPTR_DT_CANCELLATION_COUNT_ALL,
    //! Сумма аннулирований по типу чека
        LIBFPTR_DT_CANCELLATION_SUM,
    //! Сумма аннулирований всего
        LIBFPTR_DT_CANCELLATION_SUM_ALL,
    //! Состояние источника питания
        LIBFPTR_DT_POWER_SOURCE_STATE,
    //! Количество аннулирований по типу чека
        LIBFPTR_DT_CANCELLATION_COUNT,
    //! Необнуляемая сумма по типам оплат
        LIBFPTR_DT_NON_NULLABLE_SUM_BY_PAYMENTS,
    //! Температура ТПГ
        LIBFPTR_DT_PRINTER_TEMPERATURE,
    //! Фатальные ошибки
        LIBFPTR_DT_FATAL_STATUS,
    //! MAC-адрес
        LIBFPTR_DT_MAC_ADDRESS,
    //! Время работы ККТ
        LIBFPTR_DT_DEVICE_UPTIME,
    //! Количество байт, переданных в открытом чеке
        LIBFPTR_DT_RECEIPT_BYTE_COUNT,
    //! Сумма скидок и сумма надбавок за смену
        LIBFPTR_DT_DISCOUNT_AND_SURCHARGE_SUM
};

/*!
   \ingroup libfptr_group_fn

   \brief Тип данных ФН

   Список возможных значений для параметра \ref LIBFPTR_PARAM_FN_DATA_TYPE
 */
enum libfptr_fn_data_type
{
    //! Значение реквизита регистрации
        LIBFPTR_FNDT_TAG_VALUE,
    //! Статус информационного обмена с ОФД
        LIBFPTR_FNDT_OFD_EXCHANGE_STATUS,
    //! Информация об ФН
        LIBFPTR_FNDT_FN_INFO,
    //! Информация о последней регистрации/перерегистрации
        LIBFPTR_FNDT_LAST_REGISTRATION,
    //! Информация о последнем чеке
        LIBFPTR_FNDT_LAST_RECEIPT,
    //! Информация о последнем документе
        LIBFPTR_FNDT_LAST_DOCUMENT,
    //! Информация о смене
        LIBFPTR_FNDT_SHIFT,
    //! Информация о версиях ФФД
        LIBFPTR_FNDT_FFD_VERSIONS,
    //! Срок действия ФН
        LIBFPTR_FNDT_VALIDITY,
    //! Регистрационные данные
        LIBFPTR_FNDT_REG_INFO,
    //! Количество ФД за смену
        LIBFPTR_FNDT_DOCUMENTS_COUNT_IN_SHIFT,
    //! Ошибки ФН / ОФД / сети
        LIBFPTR_FNDT_ERRORS,
    //! Квитанция ОФД по номеру документа
        LIBFPTR_FNDT_TICKET_BY_DOC_NUMBER,
    //! Информация о документе по номеру
        LIBFPTR_FNDT_DOCUMENT_BY_NUMBER
};

/*!
   \ingroup libfptr_group_fn

   \brief Версия ФФД

   Список возможных значений для параметров \ref LIBFPTR_PARAM_FFD_VERSION,
    \ref LIBFPTR_PARAM_DEVICE_FFD_VERSION,
    \ref LIBFPTR_PARAM_FN_FFD_VERSION,
    \ref LIBFPTR_PARAM_DEVICE_MIN_FFD_VERSION,
    \ref LIBFPTR_PARAM_DEVICE_MAX_FFD_VERSION
 */
enum libfptr_ffd_version
{
    //! Неизвестно
        LIBFPTR_FFD_UNKNOWN = 0,
    //! ФФД 1.0
        LIBFPTR_FFD_1_0 = 100,
    //! ФФД 1.0.5
        LIBFPTR_FFD_1_0_5 = 105,
    //! ФФД 1.1
        LIBFPTR_FFD_1_1 = 110,
};

/*!
   \ingroup libfptr_group_fn, libfptr_group_registration

   \brief Система налогообложения
 */
enum libfptr_taxation_type
{
    //! По умолчанию
        LIBFPTR_TT_DEFAULT = 0x00,
    //! Общая
        LIBFPTR_TT_OSN = 0x01,
    //! Упрощенная Доход
        LIBFPTR_TT_USN_INCOME = 0x02,
    //! Упрощенная Доход минус Расход
        LIBFPTR_TT_USN_INCOME_OUTCOME = 0x04,
    //! ЕНВД
        LIBFPTR_TT_ENVD = 0x08,
    //! Единый сельскохозяйственный налог
        LIBFPTR_TT_ESN = 0x10,
    //! Панентная
        LIBFPTR_TT_PATENT = 0x20,
};

/*!
   \ingroup libfptr_group_ecr_query

   \brief Тип модуля ККТ

   Список возможных значений для параметра \ref LIBFPTR_PARAM_UNIT_TYPE
 */
enum libfptr_unit_type
{
    //! Прошивка
        LIBFPTR_UT_FIRMWARE,
    //! Конфигурация
        LIBFPTR_UT_CONFIGURATION,
    //! Движок шаблонов
        LIBFPTR_UT_TEMPLATES,
    //! Блок управления
        LIBFPTR_UT_CONTROL_UNIT,
    //! Загрузчик
        LIBFPTR_UT_BOOT,
};

/*!
   \ingroup libfptr_group_fn

   \brief Тип операции ФН

   Список возможных значений для параметра \ref LIBFPTR_PARAM_FN_OPERATION_TYPE
 */
enum libfptr_fn_operation_type
{
    //! Регистрация ФН
        LIBFPTR_FNOP_REGISTRATION = 0,
    //! Замена ФН (перерегистрация)
        LIBFPTR_FNOP_CHANGE_FN,
    //! Смена параметров (перерегистрация)
        LIBFPTR_FNOP_CHANGE_PARAMETERS,
    //! Закрытие архива ФН
        LIBFPTR_FNOP_CLOSE_ARCHIVE,
};

/*!
   \ingroup libfptr_group_fn, libfptr_group_registration

   \brief Виды агентов (реквизит 1057)
 */
enum libfptr_agent_type
{
    //! Не агент
        LIBFPTR_AT_NONE = 0x00,
    //! Банковский платежный агент
        LIBFPTR_AT_BANK_PAYING_AGENT = 0x01,
    //! Банковский платежный субагент
        LIBFPTR_AT_BANK_PAYING_SUBAGENT = 0x02,
    //! Платежный агент
        LIBFPTR_AT_PAYING_AGENT = 0x04,
    //! Платежный субагент
        LIBFPTR_AT_PAYING_SUBAGENT = 0x08,
    //! Поверенный
        LIBFPTR_AT_ATTORNEY = 0x10,
    //! Комиссионер
        LIBFPTR_AT_COMMISSION_AGENT = 0x20,
    //! Другой тип агента
        LIBFPTR_AT_ANOTHER = 0x40,
};

/*!
   \ingroup libfptr_group_lib

   \brief Канал обмена с ОФД

   Требует дополнительной настройки ККТ!
 */
enum libfptr_ofd_channel
{
    //! Не предоставлять канал обмена (в случае, если организована передача данных с помощью прямого выхода ККТ в Интернет)
        LIBFPTR_OFD_CHANNEL_NONE = 0,
    //! Предоставить для ККТ канал обмена через USB (EthernetOverUsb)
        LIBFPTR_OFD_CHANNEL_USB,
    //! Предоставить для ККТ канал обмена средствами протокола
        LIBFPTR_OFD_CHANNEL_PROTO
};

/*!
   \ingroup libfptr_group_ecr_query

   \brief Тип источника питания

   Список возможных значений для \ref LIBFPTR_PARAM_POWER_SOURCE_TYPE
 */
enum libfptr_power_source_type
{
    //! Внешний блок питания
        LIBFPTR_PST_POWER_SUPPLY = 0,
    //! Батарея часов
        LIBFPTR_PST_RTC_BATTERY,
    //! Встроенные аккумуляторы
        LIBFPTR_PST_BATTERY
};

/*!
   \ingroup libfptr_group_read_data

   \brief Тип группы данных

   Список возможных значений для \ref LIBFPTR_PARAM_RECORDS_TYPE
 */
enum libfptr_records_type
{
    //! Строки последнего напечатанного документа
        LIBFPTR_RT_LAST_DOCUMENT_LINES,
    //! Реквизиты документа по номеру из ФН в виде TLV
        LIBFPTR_RT_FN_DOCUMENT_TLVS,
    //! Результат выполнения пользовательского скрипта
        LIBFPTR_RT_EXEC_USER_SCRIPT
};

/*!
   \ingroup libfptr_group_utils

   \brief Тип кода товарной номенклатуры

   Список возможных значений для \ref LIBFPTR_PARAM_NOMENCLATURE_TYPE
 */
enum libfptr_nomenclature_type
{
    //! Маркировка изделий из меха
        LIBFPTR_NT_FURS = 0,
    //! Маркировка лекарственных препаратов
        LIBFPTR_NT_MEDICINES,
    //! Маркировка табачной продукции
        LIBFPTR_NT_TOBACCO,
};

/*!
   \ingroup libfptr_group_fn

   \brief Тип документа из ФН

   Список возможных значений для \ref LIBFPTR_PARAM_FN_DOCUMENT_TYPE
 */
enum libfptr_fn_document_type
{
    //! Отчет о регистрации
        LIBFPTR_FN_DOC_REGISTRATION = 1,
    //! Отчет об открытии смены
        LIBFPTR_FN_DOC_OPEN_SHIFT = 2,
    //! Кассовый чек
        LIBFPTR_FN_DOC_RECEIPT = 3,
    //! Бланк строгой отчетности
        LIBFPTR_FN_DOC_BSO = 4,
    //! Отчет о закрытии смены
        LIBFPTR_FN_DOC_CLOSE_SHIFT = 5,
    //! Отчет о закрытии фискального накопителя
        LIBFPTR_FN_DOC_CLOSE_FN = 6,
    //! Подтверждение оператора
        LIBFPTR_FN_DOC_OPERATOR_CONFIRMATION = 7,
    //! Отчет об изменении параметров регистрации
        LIBFPTR_FN_DOC_REREGISTRATION = 11,
    //! Отчет о текущем состоянии расчетов
        LIBFPTR_FN_DOC_EXCHANGE_STATUS = 21,
    //! Кассовый чек коррекции
        LIBFPTR_FN_DOC_CORRECTION = 31,
    //! Бланк строгой отчетности коррекции
        LIBFPTR_FN_DOC_BSO_CORRECTION = 41,
};

/*!
   \ingroup libfptr_group_utils

   \brief Уровень лога
 */
enum libfptr_log_level
{
    //! Ошибка
        LIBFPTR_LOG_ERROR = 0,
    //! Предупреждение
        LIBFPTR_LOG_WARN,
    //! Информация
        LIBFPTR_LOG_INFO,
    //! Отладка
        LIBFPTR_LOG_DEBUG
};

/*!
   \ingroup libfptr_group_user_memory

   \brief Тип операции с пользовательской памятью ККТ

   Список возможных значений для \ref LIBFPTR_PARAM_USER_MEMORY_OPERATION
 */
enum libfptr_user_memory_operation
{
    //! Запрос размера памяти
        LIBFPTR_UMO_GET_SIZE,
    //! Чтение блока данных из памяти
        LIBFPTR_UMO_READ_DATA,
    //! Запись блока данных в память
        LIBFPTR_UMO_WRITE_DATA,
    //! Чтение строки из памяти
        LIBFPTR_UMO_READ_STRING,
    //! Запись строки в память
        LIBFPTR_UMO_WRITE_STRING,
    //! Сохранение изменений в постоянную память
        LIBFPTR_UMO_COMMIT,
};

DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_init_library(void *params);

/*!
   \ingroup libfptr_group_lib

   \brief Возвращает версию библиотеки как строку.

   \return Строка с версией библиотеки
 */
DTOX_SHARED_EXPORT const char *DTOX_SHARED_CCA libfptr_get_version_string();

/*!
   \ingroup libfptr_group_lib

   \brief Создает дескриптор для работы с ККТ.

   Создает дескриптор \a handle для работы с ККТ, служащий для идентификации устройства внутри библиотеки.
   Созданный дескриптор используется во всем методах библиотеки, связанным с работой с ККТ.
   Память для дескриптора выделяется внутри библиотеки. Освободить её нужно с помощью \ref libfptr_destroy().

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_create(libfptr_handle *handle);

/*!
   \ingroup libfptr_group_lib

   \brief Настраивает дескриптор для работы с ККТ.

   При установке соединения с ККТ будут использованы настройки \a settings. Также \a settings содержат настройки дескриптора.

   Формат настроек - JSON (подробнее на странице \ref libfptr_page_settings)

   \param handle Дескриптор
   \param settings Настройки
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_set_settings(libfptr_handle handle,
                                                            const wchar_t *settings);

/*!
   \ingroup libfptr_group_lib

   \brief Удаляет дескриптор для работы с ККТ.

   \param handle Дескриптор
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_destroy(libfptr_handle *handle);

/*!
   \ingroup libfptr_group_lib

   \brief Возвращает текущие настройки дескриптора.

   \param handle Дескриптор
   \param value Буфер для строки с настройками
   \param size Размер буфера
   \return Требуемый размер буфера (с учетом '\0')
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_get_settings(libfptr_handle handle, wchar_t *value,
                                                            int size);

/*!
   \ingroup libfptr_group_lib

   \brief Возвращает значение указанной настройки дескриптора.

   Пример использования:
   \code{.cpp}
    std::vector<wchar_t> ip_addr(1024);
    int size = libfptr_get_single_setting(handle, LIBFPTR_SETTING_IPADDRESS, &ip_addr[0], ip_addr.size());
    if (size > mapping.length())
    {
        ip_addr.resize(size);
        libfptr_get_single_setting(handle, LIBFPTR_SETTING_IPADDRESS, &ip_addr[0], ip_addr.size());
    }
    std::wcout << std::wstring(&ip_addr[0]) << std::endl;
   \endcode

   \param handle Дескриптор
   \param key Название настройки
   \param value Буфер для строки со значением настройки
   \param size Размер буфера
   \return Требуемый размер буфера (с учетом '\0')
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_get_single_setting(libfptr_handle handle,
                                                                  const wchar_t *key,
                                                                  wchar_t *value, int size);

/*!
   \ingroup libfptr_group_lib

   \brief Устанавливает значение указанной настройки дескриптору.

   Указанные таким образом настройки применятся после вызова \ref libfptr_apply_single_settings().

   Пример использования приведен в описании \ref libfptr_apply_single_settings().

   \param handle Дескриптор
   \param key Название настройки
   \param value Значение настройки
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_set_single_setting(libfptr_handle handle,
                                                                   const wchar_t *key,
                                                                   const wchar_t *value);

/*!
   \ingroup libfptr_group_lib

   \brief Применяет настройки дескриптора.

   Применяет настройки, переданные с помощью \ref libfptr_set_single_setting(), к дескриптору.
   Вызов метода вызывает отключение от ККТ и повторную установку связи, если соединения до вызова
    метода было установлено.

   Пример использования:
   \code{.cpp}
    libfptr_set_single_setting(handle, LIBFPTR_SETTING_PORT, L"2"); // LIBFPTR_PORT_TCPIP
    libfptr_set_single_setting(handle, LIBFPTR_SETTING_IPADDRESS, L"192.168.5.23");
    libfptr_set_single_setting(handle, LIBFPTR_SETTING_IPPORT, L"12423");
    libfptr_apply_single_settings(handle);
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_apply_single_settings(libfptr_handle handle);

/*!
   \ingroup libfptr_group_connect

   \brief Устанавливает соединение с ККТ.

   Устанавливает соединение с ККТ по одному из каналов связи. При установке соединения используются настройки, переданные в \ref libfptr_set_settings().

   Пример использования:
   \code{.cpp}
    if (libfptr_open(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_open(libfptr_handle handle);

/*!
   \ingroup libfptr_group_connect

   \brief Разрывает соединение с ККТ.

   Пример использования:
   \code{.cpp}
    if (libfptr_close(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_close(libfptr_handle handle);

/*!
   \ingroup libfptr_group_connect

   \brief Возвращает 1, если соединение с ККТ было установлено (был вызван \ref libfptr_open()), или 0,
   если соединение было разорвано пользователем (был вызван \ref libfptr_close()), либо \ref libfptr_open() не был
   вызван.

   По результату данного метода нельзя судить о текущем состоянии соединения с ККТ.

   \param handle Дескриптор
   \retval 1 Соединение установлено
   \retval 0 Соединение не установлено
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_is_opened(libfptr_handle handle);

/*!
   \ingroup libfptr_group_errors

   \brief Возвращает код последней ошибки библиотеки.

   Возвращает код последней ошибки библиотеки, привязанный к дескриптору \a handle.
   Описание ошибки можно получить с помощью \ref libfptr_error_description().
   Имеет смысл для методов, которые вернули индикацию ошибки выполнения.

   \param handle Дескриптор
   \return Код ошибки
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_error_code(libfptr_handle handle);

/*!
   \ingroup libfptr_group_errors

   \brief Возвращает описание последней ошибки библиотеки.

   Возвращает описание последней ошибки библиотеки, привязанный к дескриптору \a handle.
   Код ошибки можно получить с помощью \ref libfptr_error_code().
   Имеет смысл для методов, которые вернули индикацию ошибки выполнения.

   \param handle Дескриптор
   \param value Буфер для описания ошибки
   \param size Размер буфера
   \return Требуемый размер буфера (с учетом '\0')
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_error_description(libfptr_handle handle,
                                                                 wchar_t *value, int size);

/*!
   \ingroup libfptr_group_params

   \brief Устанавливает значение параметра.

   Устанавливает параметру \a param_id (\ref libfptr_param) логическое значение \a value.
   Значение параметра будет проверено и применено при ближайшем вызове метода работы с ККТ, после чего будет удалено.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param value Да/Нет
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_set_param_bool(libfptr_handle handle, int param_id,
                                                               int value);

/*!
   \ingroup libfptr_group_params

   \brief Устанавливает значение параметра.

   Устанавливает параметру \a param_id (\ref libfptr_param) целочисленное значение \a value.
   Значение параметра будет проверено и применено при ближайшем вызове метода работы с ККТ, после чего будет удалено.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param value Число
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_set_param_int(libfptr_handle handle, int param_id,
                                                              uint value);

/*!
   \ingroup libfptr_group_params

   \brief Устанавливает значение параметра.

   Устанавливает параметру \a param_id (\ref libfptr_param) значение с плавающей точкой \a value.
   Значение параметра будет проверено и применено при ближайшем вызове метода работы с ККТ, после чего будет удалено.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param value Число
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_set_param_double(libfptr_handle handle,
                                                                 int param_id,
                                                                 double value);

/*!
   \ingroup libfptr_group_group_params

   \brief Устанавливает значение параметра.

   Устанавливает параметру \a param_id (\ref libfptr_param) строковое значение \a value.
   Значение параметра будет проверено и применено при ближайшем вызове метода работы с ККТ, после чего будет удалено.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param value Строка
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_set_param_str(libfptr_handle handle, int param_id,
                                                              const wchar_t *value);

/*!
   \ingroup libfptr_group_params

   \brief Устанавливает значение параметра.

   Устанавливает параметру \a param_id (\ref libfptr_param) значение даты (\a year, \a month, \a day) и времени (\a hour, \a minute, \a second).
   Значение параметра будет проверено и применено при ближайшем вызове метода работы с ККТ, после чего будет удалено.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param year Год
   \param month Месяц
   \param day День
   \param hour Час
   \param minute Минута
   \param second Секунда
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_set_param_datetime(libfptr_handle handle,
                                                                   int param_id,
                                                                   int year, int month, int day,
                                                                   int hour, int minute,
                                                                   int second);

/*!
   \ingroup libfptr_group_params

   \brief Устанавливает значение параметра.

   Устанавливает параметру \a param_id (\ref libfptr_param) байтовый массив \a value размером \a size.
   Значение параметра будет проверено и применено при ближайшем вызове метода работы с ККТ, после чего будет удалено.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param value Буфер
   \param size Размер буфера
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_set_param_bytearray(libfptr_handle handle,
                                                                    int param_id,
                                                                    const uchar *value, int size);

/*!
   \ingroup libfptr_group_params

   \brief Возвращает значение параметра.

   Возвращает логическое представление параметра \a param_id.

   \param handle Дескриптор
   \param param_id Номер параметра
   \retval 1 - true, 0 - false
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_get_param_bool(libfptr_handle handle, int param_id);

/*!
   \ingroup libfptr_group_params

   \brief Возвращает значение параметра.

   Возвращает целочисленное представление параметра \a param_id.

   \param handle Дескриптор
   \param param_id Номер параметра
   \retval Число
 */
DTOX_SHARED_EXPORT uint DTOX_SHARED_CCA libfptr_get_param_int(libfptr_handle handle, int param_id);

/*!
   \ingroup libfptr_group_params

   \brief Возвращает значение параметра.

   Возвращает представленное в виде значения с плавающей точкой значение параметра \a param_id.

   \param handle Дескриптор
   \param param_id Номер параметра
   \retval Число
 */
DTOX_SHARED_EXPORT double DTOX_SHARED_CCA libfptr_get_param_double(libfptr_handle handle,
                                                                   int param_id);

/*!
   \ingroup libfptr_group_params

   \brief Возвращает значение параметра.

   Возвращает строковое представление параметра \a param_id в \a value.
   Если данный параметр недоступен совсем или имеет несовместимый тип, то вернется ошибка.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param value Строка
   \param value Буфер для строки
   \param size Размер буфера
   \return Требуемый размер буфера (с учетом '\0')
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_get_param_str(libfptr_handle handle, int param_id,
                                                             wchar_t *value, int size);

/*!
   \ingroup libfptr_group_params

   \brief Возвращает значение параметра.

   Возвращает представление параметра \a param_id как даты и времени.
   Если данный параметр недоступен совсем или имеет несовместимый тип, то вернется ошибка.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param year Год
   \param month Месяц
   \param day День
   \param hour Час
   \param minute Минута
   \param second Секунда
 */
DTOX_SHARED_EXPORT void DTOX_SHARED_CCA libfptr_get_param_datetime(libfptr_handle handle,
                                                                   int param_id,
                                                                   int *year, int *month, int *day,
                                                                   int *hour, int *minute,
                                                                   int *second);

/*!
   \ingroup libfptr_group_params

   \brief Возвращает значение параметра.

   Возвращает значение параметра \a param_id в виде байтового массива \a value.
   Если данный параметр недоступен совсем или имеет несовместимый тип, то вернется ошибка.

   \param handle Дескриптор
   \param param_id Номер параметра
   \param value Буфер для значения
   \param size Размер буфера
   \return Требуемый размер буфера
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_get_param_bytearray(libfptr_handle handle,
                                                                   int param_id,
                                                                   uchar *value, int size);

/*!
   \ingroup libfptr_group_params

   \brief Сбрасывает значение всех установленных параметров.

   \param handle Дескриптор
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_reset_params(libfptr_handle handle);

/*!
   \ingroup libfptr_group_misc

   \brief Выполняет команду ККТ.

   Выполняет произвольную команду по протоколу работы текущей настроенной ККТ.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_TIMEOUT_ENQ      <td>Таймаут ожидания ответа ККТ, мс.    <td>int <td>5000
    <tr><td>\ref LIBFPTR_PARAM_NO_NEED_ANSWER   <td>Не ждать ответа от ККТ              <td>bool <td>false
    <tr><td>\ref LIBFPTR_PARAM_COMMAND_BUFFER   <td>Буфер с командой ККТ                <td>bytearray <td>+
    <tr><th colspan="4">Выходные параметры
    <tr><td>\ref LIBFPTR_PARAM_ANSWER_BUFFER    <td>Буфер с ответом ККТ                 <td colspan="2">bytearray
   </table>

   Пример использования:
   \code{.cpp}
    uchar command[4] = {0x91, 0x13, 0x00, 0x00};
    std::vector<uchar> answer;

    libfptr_set_param_int(handle, LIBFPTR_PARAM_TIMEOUT_ENQ, 10000);
    libfptr_set_param_bytearray(handle, LIBFPTR_PARAM_COMMAND_BUFFER, command, sizeof(command) / sizeof(command[0]));
    if (libfptr_run_command(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    int size = libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_ANSWER_BUFFER, &answer[0], answer.size());
    if (size > answer.size())
    {
        answer.resize(size);
        libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_ANSWER_BUFFER, &answer[0], answer.size());
    }

    // Обработка answer
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_run_command(libfptr_handle handle);

/*!
   \ingroup libfptr_group_misc

   \brief Подает звуковой сигнал с помощью ККТ.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_FREQUENCY        <td>Частота сигнала, Гц.        <td>int <td>-
    <tr><td>\ref LIBFPTR_PARAM_DURATION         <td>Длительность сигнала, мс.   <td>int <td>-
   </table>

   Если ККТ не может подавать сложный звуковой сигнал, подается простой гудок.

   Пример использования:
   \code{.cpp}
    // Сложный звуковой сигнал
    libfptr_set_param_int(handle, LIBFPTR_PARAM_FREQUENCY, 5000);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_DURATION, 1000);
    if (libfptr_beep(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Простой звуковой сигнал (гудок)
    if (libfptr_beep(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_beep(libfptr_handle handle);

/*!
   \ingroup libfptr_group_misc

   \brief Открывает денежный ящик, подключенный к ККТ.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_DRAWER_ON_TIMEOUT    <td>Время включения ДЯ, мс      <td>int <td>-
    <tr><td>\ref LIBFPTR_PARAM_DRAWER_OFF_TIMEOUT   <td>Время выключения ДЯ, мс.    <td>int <td>-
    <tr><td>\ref LIBFPTR_PARAM_DRAWER_ON_QUANTITY   <td>Количество импульсов        <td>int <td>-
   </table>

   Если ККТ не может произвести продвинутое открытие денежного ящика, то производится простое

   Пример использования:
   \code{.cpp}
    // Продвинутое открытие денежного ящика
    libfptr_set_param_int(handle, LIBFPTR_PARAM_DRAWER_ON_TIMEOUT, 5000);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_DRAWER_OFF_TIMEOUT, 5000);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_DRAWER_ON_QUANTITY, 2);
    if (libfptr_open_drawer(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Простое открытие денежного ящика
    if (libfptr_open_drawer(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех

   \warning По результатам выполнения данного метода нельзя судить о:
   - наличии связи с денежным ящиком;
   - состоянии ящика (открылся ящик или нет)
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_open_drawer(libfptr_handle handle);

/*!
   \ingroup libfptr_group_misc

   \brief Отрезает чековую ленту.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_CUT_TYPE                 <td>Способ отрезки чековой ленты    <td>int (\ref libfptr_cut_type) <td>При отсутствии параметра библиотека перебирает способы отрезки
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_CUT_TYPE, LIBFPTR_CT_FULL);
    if (libfptr_cut(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_cut(libfptr_handle handle);

/*!
   \ingroup libfptr_group_service

   \brief Производит программное выключение ККТ.

   Пример использования:
   \code{.cpp}
    if (libfptr_poweroff(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_device_poweroff(libfptr_handle handle);

/*!
   \ingroup libfptr_group_service

   \brief Производит перезагрузку ККТ.

   Пример использования:
   \code{.cpp}
    if (libfptr_reboot(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_device_reboot(libfptr_handle handle);

/*!
   \ingroup libfptr_group_report

   \brief Открывает смену в ККТ.

   Возвращает ошибку, если смена уже открыта.
   Метод необязательный, т.к. первая фискальная операция в открывает смену автоматически.

   \warning Требует проверки результата закрытия документа - \ref libfptr_check_document_closed().

   Пример использования:
   \code{.cpp}
    if (libfptr_open_shift(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_open_shift(libfptr_handle handle);

/*!
   \ingroup libfptr_group_service

   \brief Производит общее гашение ККТ.

   Пример использования:
   \code{.cpp}
    if (libfptr_reset_summary(handle) < 0)
    {
        // Обработка ошибки
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_reset_summary(libfptr_handle handle);

/*!
   \ingroup libfptr_group_service

   \brief Производит начальную инициализацию ККТ.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_SERIAL_NUMBER        <td>Серийный номер ККТ     <td>string  <td>
    <tr><td>\ref LIBFPTR_PARAM_MAC_ADDRESS          <td>MAC-адрес               <td>string  <td>
    <tr><td>\ref LIBFPTR_PARAM_DATE_TIME            <td>Текущие дата и время    <td>string  <td>
    <tr><td>\ref LIBFPTR_PARAM_FULL_RESET           <td>Выполнять полную очистку памяти    <td>bool  <td>false
   </table>

   Библиотека сама добавляет ведущие нули к серийному номеру.
   Если \ref LIBFPTR_PARAM_DATE_TIME не передан, вводится текущее время хоста.

   Пример использования:
   \code{.cpp}

    libfptr_set_param_str(handle, LIBFPTR_PARAM_SERIAL_NUMBER, L"106300000011");
    libfptr_set_param_str(handle, LIBFPTR_PARAM_MAC_ADDRESS, L"106300000011");
    libfptr_set_param_datetime(handle, LIBFPTR_PARAM_DATE_TIME, 2017, 1, 2, 12, 13, 45);
    if (libfptr_init_device(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_init_device(libfptr_handle handle);

/*!
   \ingroup libfptr_group_ecr_query

   \brief Запрашивает данные из ККТ

   Все входящие параметры обязательны.

   <table>
    <caption>Список данных</caption>
    <tr><th rowspan="2">Тип данных (LIBFPTR_PARAM_DATA_TYPE)       <th rowspan="2">Описание    <th colspan="3">Входные параметры   <th colspan="3">Выходные параметры
    <tr><th>Название<th>Описание<th>Тип параметра<th>Название<th>Описание<th>Тип параметра
    <tr><td rowspan="30">\ref LIBFPTR_DT_STATUS             <td rowspan="30">Общая информация и статус ККТ
    <tr><td rowspan="29" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_OPERATOR_ID              <td>Номер кассира                       <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_LOGICAL_NUMBER           <td>Номер ККТ в магазине                <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_DATE_TIME                <td>Дата и время ККТ                    <td>datetime
    <tr>            <td>\ref LIBFPTR_PARAM_FISCAL                   <td>ККТ фискализирована                 <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_FN_FISCAL                <td>ФН фискализирован                   <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_SHIFT_STATE              <td>Состояние смены                     <td>int (\ref libfptr_shift_state)
    <tr>            <td>\ref LIBFPTR_PARAM_CASHDRAWER_OPENED        <td>Денежный ящик открыт                <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_PAPER_PRESENT    <td>Есть бумага                         <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_COVER_OPENED             <td>Крышка открыта                      <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_SERIAL_NUMBER            <td>Серийный номер ККТ                   <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_MODEL                    <td>Номер модели ККТ                    <td>int (\ref libfptr_model)
    <tr>            <td>\ref LIBFPTR_PARAM_MODE                     <td>Режим ККТ                           <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_SUBMODE                  <td>Подрежим ККТ                        <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_NUMBER           <td>Номер чека (внутренний счетчик ККТ) <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER          <td>Номер документа (внутренний счетчик ККТ) <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_SHIFT_NUMBER             <td>Номер открытой смены или номер последней закрытой смены + 1 <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип открытого чека                  <td>int (\ref libfptr_receipt_type)
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_SUM              <td>Сумма текущего чека, р              <td>double
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_LINE_LENGTH      <td>Ширина чековой ленты, симв.         <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_LINE_LENGTH_PIX  <td>Ширина чековой ленты, пикс.         <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_MODEL_NAME               <td>Название ККТ                        <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_UNIT_VERSION             <td>Версия ПО ККТ                       <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_PRINTER_CONNECTION_LOST  <td>Потеряно соединение с принтером     <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_PRINTER_ERROR            <td>Невосстановимая ошибка принтера     <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_CUT_ERROR                <td>Ошибка отрезчика                    <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_PRINTER_OVERHEAT         <td>Перегрев ТПГ                        <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_FN_PRESENT               <td>Флаг присутствия ФН                 <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_BLOCKED                  <td>Блокировка ККТ                      <td>bool

    <tr><td>\ref LIBFPTR_DT_CASH_SUM            <td>Сумма наличных в денежном ящике
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_SUM                      <td>Сумма, р.                   <td>double

    <tr><td>\ref LIBFPTR_DT_UNIT_VERSION        <td>Версия модуля
                    <td>\ref LIBFPTR_PARAM_UNIT_TYPE                <td>Тип модуля                  <td>int (\ref libfptr_unit_type)
                    <td>\ref LIBFPTR_PARAM_UNIT_VERSION             <td>Версия модуля               <td>string

    <tr><td rowspan="2">\ref LIBFPTR_DT_PICTURE_INFO        <td rowspan="2">Параметры картинки в памяти / внешнем хранилище
        <td rowspan="2">\ref LIBFPTR_PARAM_PICTURE_NUMBER           <td rowspan="2">Номер картинки  <td rowspan="2">int
                    <td>\ref LIBFPTR_PARAM_HEIGHT                   <td>Высота картинки, пикс.      <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_WIDTH                    <td>Ширина картинки, пикс.      <td>int

    <tr><td>\ref LIBFPTR_DT_LICENSE_ACTIVATED               <td>Состояние кода защиты
                    <td>\ref LIBFPTR_PARAM_LICENSE_NUMBER           <td>Номер кода защиты               <td>int
                    <td>\ref LIBFPTR_PARAM_LICENSE_ENTERED          <td>Состояние кода защиты           <td>bool

    <tr><td>\ref LIBFPTR_DT_REGISTRATIONS_SUM               <td >Сумма регистраций
                    <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека                        <td>int (\ref libfptr_receipt_type)
                    <td>\ref LIBFPTR_PARAM_SUM                      <td>Сумма, р.                       <td>double

    <tr><td>\ref LIBFPTR_DT_REGISTRATIONS_COUNT             <td>Количество регистраций
                    <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека                        <td>int (\ref libfptr_receipt_type)
                    <td>\ref LIBFPTR_PARAM_COUNT                    <td>Количество                      <td>int

    <tr><td rowspan="3">\ref LIBFPTR_DT_PAYMENT_SUM         <td rowspan="3">Сумма платежей
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека                        <td>int
                    <td rowspan="2">\ref LIBFPTR_PARAM_SUM          <td rowspan="2">Сумма, р.           <td rowspan="2">double
    <tr>            <td>\ref LIBFPTR_PARAM_PAYMENT_TYPE             <td>Тип оплаты                      <td>int (\ref libfptr_payment_type)

    <tr><td>\ref LIBFPTR_DT_CASHIN_SUM                      <td>Сумма внесений
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_SUM                      <td>Сумма, р.       <td>double

    <tr><td>\ref LIBFPTR_DT_CASHIN_COUNT                    <td>Количество внесений
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_DOCUMENTS_COUNT          <td>Количество      <td>int

    <tr><td>\ref LIBFPTR_DT_CASHOUT_SUM                     <td>Сумма выплат
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_SUM                      <td>Сумма, р.       <td>double

    <tr><td>\ref LIBFPTR_DT_CASHOUT_COUNT                   <td>Количество выплат
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_DOCUMENTS_COUNT          <td>Количество      <td>int

    <tr><td>\ref LIBFPTR_DT_REVENUE                         <td>Выручка
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_SUM                      <td>Сумма, р.       <td>double

    <tr><td>\ref LIBFPTR_DT_DATE_TIME                       <td>Текущие дата и время в ККТ
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_DATE_TIME                <td>Дата и время    <td>datetime

    <tr><td rowspan="5">\ref LIBFPTR_DT_SHIFT_STATE         <td rowspan="5">Состояние смены
    <tr><td rowspan="4" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_SHIFT_STATE              <td>Состояние смены                 <td>int (\ref libfptr_shift_state)
    <tr>            <td>\ref LIBFPTR_PARAM_DATE_TIME                <td>Дата и время окончания смены    <td>datetime
    <tr>            <td>\ref LIBFPTR_PARAM_SHIFT_NUMBER             <td>Номер смены                     <td>int

    <tr><td rowspan="8">\ref LIBFPTR_DT_RECEIPT_STATE       <td rowspan="8">Состояние чека
    <tr><td rowspan="7" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека                        <td>int (\ref libfptr_receipt_type)
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_NUMBER           <td>Номер чека                      <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER          <td>Номер документа                 <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_SUM              <td>Сумма открытого чека            <td>double
    <tr>            <td>\ref LIBFPTR_PARAM_REMAINDER                <td>Неоплаченный остаток чека, р.   <td>double
    <tr>            <td>\ref LIBFPTR_PARAM_CHANGE                   <td>Сдача, р.                       <td>double

    <tr><td>\ref LIBFPTR_DT_SERIAL_NUMBER                   <td>Серийный номер
                <td colspan="3">
                <td>\ref LIBFPTR_PARAM_SERIAL_NUMBER            <td>Серийный номер                 <td>string

    <tr><td rowspan="5">\ref LIBFPTR_DT_MODEL_INFO          <td rowspan="5">Информация о ККТ
    <tr><td rowspan="4" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_MODEL                    <td>Номер модели ККТ                <td>int (\ref libfptr_model)
    <tr>            <td>\ref LIBFPTR_PARAM_UNIT_VERSION             <td>Версия ПО ККТ                   <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_MODEL_NAME               <td>Название ККТ                    <td>string

    <tr><td rowspan="4">\ref LIBFPTR_DT_RECEIPT_LINE_LENGTH <td rowspan="4">Ширина ленты
    <tr><td rowspan="3" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_LINE_LENGTH      <td>Ширина чековой ленты, симв.     <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_LINE_LENGTH_PIX  <td>Ширина чековой ленты, пикс.     <td>int

    <tr><td>\ref LIBFPTR_DT_CUTTER_RESOURCE                 <td>Ресурс отрезчика
                    <td>\ref LIBFPTR_PARAM_COUNTER_TYPE             <td rowspan="2">Тип счетчика    <td rowspan="2">int (\ref libfptr_counter_type)
                    <td>\ref LIBFPTR_PARAM_COUNT                    <td>Количество                  <td>int

    <tr><td rowspan="3">\ref LIBFPTR_DT_STEP_RESOURCE       <td rowspan="3">Ресурс ШД
    <tr>            <td>\ref LIBFPTR_PARAM_COUNTER_TYPE             <td>Тип счетчика                <td>int (\ref libfptr_counter_type)
                    <td rowspan="2">\ref LIBFPTR_PARAM_COUNT        <td rowspan="2">Количество      <td rowspan="2">int
    <tr>            <td>\ref LIBFPTR_PARAM_STEP_COUNTER_TYPE        <td>Тип счетчика шагов          <td>int (\ref libfptr_step_counter_type)

    <tr><td>\ref LIBFPTR_DT_TERMAL_RESOURCE                 <td>Ресурс ТПГ
                <td>\ref LIBFPTR_PARAM_COUNTER_TYPE             <td>Тип счетчика                <td>int (\ref libfptr_counter_type)
                <td>\ref LIBFPTR_PARAM_COUNT                    <td>Количество                  <td>int

    <tr><td rowspan="3">\ref LIBFPTR_DT_SHIFT_TAX_SUM       <td rowspan="3">Сумма налога за смену
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека                        <td>int (\ref libfptr_receipt_type)
                    <td rowspan="2">\ref LIBFPTR_PARAM_SUM          <td rowspan="2">Сумма налога, р.    <td rowspan="2">double
    <tr>            <td>\ref LIBFPTR_PARAM_TAX_TYPE                 <td>Ставка налога                   <td>int (\ref libfptr_tax_type)

    <tr><td>\ref LIBFPTR_DT_RECEIPT_TAX_SUM                 <td>Сумма налога на чек
                <td>\ref LIBFPTR_PARAM_TAX_TYPE                     <td>Ставка налога               <td>int (\ref libfptr_tax_type)
                <td>\ref LIBFPTR_PARAM_SUM                          <td>Сумма налога, р.            <td>double

    <tr><td>\ref LIBFPTR_DT_NON_NULLABLE_SUM                <td>Необнуляемая сумма
                <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE                 <td>Тип чека                    <td>int (\ref libfptr_receipt_type)
                <td>\ref LIBFPTR_PARAM_SUM                          <td>Необнуляемая сумма, р.      <td>double

    <tr><td>\ref LIBFPTR_DT_RECEIPT_COUNT                   <td>Количество чеков
                <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека        <td>int (\ref libfptr_receipt_type)
                <td>\ref LIBFPTR_PARAM_DOCUMENTS_COUNT          <td>Количество      <td>int

    <tr><td>\ref LIBFPTR_DT_CANCELLATION_COUNT_ALL          <td>Количество аннулирований всего
                <td colspan="3">
                <td>\ref LIBFPTR_PARAM_DOCUMENTS_COUNT          <td>Количество      <td>int

    <tr><td>\ref LIBFPTR_DT_CANCELLATION_SUM                <td>Сумма аннулирований по типу чека
                <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека        <td>int (\ref libfptr_receipt_type)
                <td>\ref LIBFPTR_PARAM_SUM                      <td>Сумма, р.       <td>double

    <tr><td>\ref LIBFPTR_DT_CANCELLATION_SUM_ALL            <td>Сумма аннулирований всего
                <td colspan="3">
                <td>\ref LIBFPTR_PARAM_SUM                      <td>Сумма, р.       <td>double

    <tr><td rowspan="5">\ref LIBFPTR_DT_POWER_SOURCE_STATE   <td rowspan="5">Состояние источника питания
                <td rowspan="5">\ref LIBFPTR_PARAM_POWER_SOURCE_TYPE        <td rowspan="5">Тип источника    <td rowspan="5">int (\ref libfptr_power_source_type)
                <td>\ref LIBFPTR_PARAM_BATTERY_CHARGE           <td>Заряд аккумулятора, процент <td>int
    <tr>        <td>\ref LIBFPTR_PARAM_VOLTAGE                  <td>Напряжение источника питания, В <td>double
    <tr>        <td>\ref LIBFPTR_PARAM_USE_BATTERY              <td>Устройство работает от аккумулятора <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_BATTERY_CHARGING         <td>Аккумуляторы заряжаются <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_CAN_PRINT_WHILE_ON_BATTERY <td>Возможность печати при текущем заряде аккумулятора <td>bool

    <tr><td>\ref LIBFPTR_DT_CANCELLATION_COUNT              <td>Количество аннулирований по типу чека
                <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека        <td>int (\ref libfptr_receipt_type)
                <td>\ref LIBFPTR_PARAM_DOCUMENTS_COUNT          <td>Количество      <td>int

    <tr><td rowspan="2">\ref LIBFPTR_DT_NON_NULLABLE_SUM_BY_PAYMENTS    <td rowspan="2">Необнуляемая сумма по типам оплат
                   <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека                        <td>int
                   <td rowspan="2">\ref LIBFPTR_PARAM_SUM          <td rowspan="2">Сумма, р.           <td rowspan="2">double
    <tr>           <td>\ref LIBFPTR_PARAM_PAYMENT_TYPE             <td>Тип оплаты                      <td>int (\ref libfptr_payment_type)

    <tr><td>\ref LIBFPTR_DT_PRINTER_TEMPERATURE             <td>Температура ТПГ
                <td colspan="3">
                <td>\ref LIBFPTR_PARAM_PRINTER_TEMPERATURE  <td>Температура ТПГ, градусы    <td>int

    <tr><td rowspan="13">\ref LIBFPTR_DT_FATAL_STATUS                    <td rowspan="13">Фатальные ошибки
                <td colspan="3" rowspan="13">
                <td>\ref LIBFPTR_PARAM_NO_SERIAL_NUMBER         <td>Не введен ЗН ККТ                    <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_RTC_FAULT                <td>Ошибка часов реального времени      <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_SETTINGS_FAULT           <td>Ошибка настроек                     <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_COUNTERS_FAULT           <td>Ошибка счетчиков                    <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_USER_MEMORY_FAULT        <td>Ошибка пользовательской памяти      <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_SERVICE_COUNTERS_FAULT   <td>Ошибка сервисных регистров          <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_ATTRIBUTES_FAULT         <td>Ошибка реквизитов                   <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_FN_FAULT                 <td>Фатальная ошибка ФН                 <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_INVALID_FN               <td>Установлен ФН из другой ККТ         <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_HARD_FAULT               <td>Фатальная аппаратная ошибка         <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_MEMORY_MANAGER_FAULT     <td>Ошибка диспетчера памяти            <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_SCRIPTS_FAULT            <td>Скрипты повреждены или отсутствуют  <td>bool
    <tr>        <td>\ref LIBFPTR_PARAM_WAIT_FOR_REBOOT          <td>Требуется перезагрузка              <td>bool

    <tr><td>\ref LIBFPTR_DT_MAC_ADDRESS                         <td>MAC-адрес
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_MAC_ADDRESS          <td>MAC-адрес       <td>string

    <tr><td>\ref LIBFPTR_DT_DEVICE_UPTIME                       <td>Время работы ККТ
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_DEVICE_UPTIME        <td>Время работы ККТ, с.        <td>int

    <tr><td>\ref LIBFPTR_DT_RECEIPT_BYTE_COUNT                  <td>Количество байт, переданных в открытом чеке
                    <td colspan="3">
                    <td>\ref LIBFPTR_PARAM_COUNT                <td>Количество байт, переданных в открытом чеке, б.        <td>int

    <tr><td rowspan="4">\ref LIBFPTR_DT_DISCOUNT_AND_SURCHARGE_SUM <td rowspan="4">Сумма скидок и сумма надбавок за смену
    <tr><td rowspan="3" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_DISCOUNT_SUM      <td>Сумма скидок за смену      <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_SURCHARGE_SUM  <td>Сумма надбавок за смену    <td>int


   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_DATA_TYPE, LIBFPTR_DT_UNIT_VERSION);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_UNIT_TYPE, LIBFPTR_UT_FIRMWARE);
    if (libfptr_query_data(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Получение результатов запроса
    std::vector<wchar_t> unitVersion(1024);
    int size = libfptr_get_param_str(handle, LIBFPTR_PARAM_UNIT_VERSION, &unitVersion[0], unitVersion.size());
    if (size > unitVersion.length())
    {
        unitVersion.resize(size);
        libfptr_get_param_str(handle, LIBFPTR_PARAM_UNIT_VERSION, &unitVersion[0], unitVersion.size());
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_query_data(libfptr_handle handle);

/*!
   \ingroup libfptr_group_cashinout

   \brief Производит внесение наличности.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_SUM         <td>Сумма внесения, р.      <td>double   <td>+
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_double(handle, LIBFPTR_PARAM_SUM, 100.50);
    if (libfptr_cash_income(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_cash_income(libfptr_handle handle);

/*!
   \ingroup libfptr_group_cashinout

   \brief Производит изъятие наличности.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_SUM          <td>Сумма изъятия, р.      <td>double <td>+
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_double(handle, LIBFPTR_PARAM_SUM, 100.50);
    if (libfptr_cash_outcome(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_cash_outcome(libfptr_handle handle);


/*!
   \ingroup libfptr_group_receipt

   \brief Открывает чек.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра  <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_RECEIPT_TYPE             <td>Тип чека            <td>int (\ref libfptr_receipt_type) <td>+
    <tr><td>\ref LIBFPTR_PARAM_RECEIPT_ELECTRONICALLY   <td>Электронный чек     <td>bool                            <td>false
    <tr><th colspan="6">Допустимые реквизиты ФН для всех чеков (чеков коррекции после ФФД 1.1)
    <tr><td>1005    <td>Адрес оператора по переводу денежных средств        <td>string<td>
    <tr><td>1008    <td>Номер телефона / email клиента                      <td>string<td>
    <tr><td>1016    <td>ИНН оператора по переводу денежных средств          <td>string<td>
    <tr><td>1021    <td>Кассир                                              <td>string<td>
    <tr><td>1026    <td>Наименование оператора по переводу денежных средств <td>string<td>
    <tr><td>1044    <td>Операция банковского агента                         <td>string<td>
    <tr><td>1055    <td>Применяемая система налогообложения                 <td>int<td>
    <tr><td>1057    <td>Признак платежного агента                           <td>int<td>
    <tr><td>1073    <td>Телефон платежного агента                           <td>string<td>
    <tr><td>1074    <td>Телефон оператора по приему платежей                <td>string<td>
    <tr><td>1075    <td>Телефон оператора по переводу денежных средств      <td>string<td>
    <tr><td>1084    <td>Дополнительный реквизит пользователя                <td>bytearray<td>
    <tr><td>1108    <td>Признак расчетов в сети Интернет                    <td>bool<td>
    <tr><td>1117    <td>Адрес электронной почты отправителя чека            <td>string<td>
    <tr><td>1171    <td>Телефон поставщика (возможна передача нескольких реквизитов)<td>string<td>
    <tr><td>1192    <td>Дополнительный реквизит чека                        <td>string<td>
    <tr><td>1203    <td>ИНН кассира                                         <td>string<td>
    <tr><td>1227    <td>Наименование клиента                                <td>string<td>
    <tr><td>1228    <td>ИНН клиента                                         <td>string<td>
    <tr><th colspan="4">Дополнительные реквизиты для чеков коррекции, помимо выше перечисленных (ФФД 1.1)
    <tr><td>1174    <td>Основание для коррекции                             <td>bytearray<td>
    <tr><th colspan="4">Допустимые реквизиты ФН для чеков коррекции (ФФД 1.0, ФФД 1.05)
    <tr><td>1021    <td>Кассир                                              <td>string<td>
    <tr><td>1055    <td>Применяемая система налогообложения                 <td>int<td>
    <tr><td>1173    <td>Тип коррекции                                       <td>int<td>
    <tr><td>1174    <td>Основание для коррекции                             <td>bytearray<td>
    <tr><td>1203    <td>ИНН кассира                                         <td>string<td>
   </table>
   <br>
   <table>
    <caption>Описание реквизита 1084 (Дополнительный реквизит пользователя) </caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра
    <tr><td>1085    <td>Наименование дополнительного реквизита              <td>string
    <tr><td>1086    <td>Значение дополнительного реквизита                  <td>string
   </table>
   <br>
   <table>
    <caption>Описание реквизита 1174 (Основание для коррекции) (ФФД 1.0, ФФД 1.05)</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра
    <tr><td>1177    <td>Наименование основания для коррекции                <td>string
    <tr><td>1178    <td>Дата документа основания для коррекции              <td>datetime
    <tr><td>1179    <td>Номер документа основания для коррекции             <td>string
   </table>
   <br>
   <table>
    <caption>Описание реквизита 1174 (Основание для коррекции) (ФФД 1.1)</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра
    <tr><td>1173    <td>Тип коррекции                                       <td>int
    <tr><td>1177    <td>Наименование основания для коррекции                <td>string
    <tr><td>1178    <td>Дата документа основания для коррекции              <td>datetime
    <tr><td>1179    <td>Номер документа основания для коррекции             <td>string
   </table>

   \warning Печать электронного чека требует установленного параметра \ref LIBFPTR_PARAM_RECEIPT_ELECTRONICALLY и реквизита 1008 (адрес получателя)

   Если один из реквизитов ФН не удалось запрограммировать, ошибка возвращена не будет (чек откроется),
    но будет параметр \ref LIBFPTR_PARAM_ERROR_TAG_NUMBER будет инициализирован номером реквизита,
    запись которого была завершена с ошибкой.

   \warning Для регистрации кассира рекомендуется использовать \ref libfptr_operator_login()

   Пример использования:
   \code{.cpp}
    // Обычный чек
    libfptr_set_param_int(handle, LIBFPTR_PARAM_RECEIPT_TYPE, LIBFPTR_RT_SELL);
    if (libfptr_open_receipt(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Печать чека + отправка пользователю на указанный контакт (с помощью ОФД)
    libfptr_set_param_int(handle, LIBFPTR_PARAM_RECEIPT_TYPE, LIBFPTR_RT_SELL);
    libfptr_set_param_str(handle, 1008, L"user@test.ru");
    if (libfptr_open_receipt(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Отправка чека пользователю на указанный контакт (с помощью ОФД) без печати на чековой ленте
    libfptr_set_param_int(handle, LIBFPTR_PARAM_RECEIPT_TYPE, LIBFPTR_RT_SELL);
    libfptr_set_param_str(handle, 1008, L"user@test.ru");
    libfptr_set_param_bool(handle, LIBFPTR_PARAM_RECEIPT_ELECTRONICALLY, true);
    if (libfptr_open_receipt(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_open_receipt(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Отменяет открытый чек.

   Пример использования:
   \code{.cpp}
    if (libfptr_cancel_receipt(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_cancel_receipt(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Закрывает открытый чек.

   Закрывает чек без сдачи.
   Если чек уже частично оплачен, зафиксирует оплату остатка заданным типом. Если чек оплачен полностью, передача \ref LIBFPTR_PARAM_PAYMENT_TYPE
    ни на что не влияет.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_PAYMENT_TYPE     <td>Тип оплаты      <td>int (\ref libfptr_payment_type) <td>\ref LIBFPTR_PT_CASH
   </table>

   \warning Требует проверки результата закрытия документа - \ref libfptr_check_document_closed().

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_PAYMENT_TYPE, LIBFPTR_PT_CASH);
    if (libfptr_close_receipt(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_close_receipt(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Проверяет результат закрытия фискального документа.

   В ряде ситуаций (окончание бумаги, потеря связи с ККТ в момент регистрации документа) состояние документа остается неизвестным.
   Данный метод сверяет счетчики ККТ с сохраненными до закрытия документа копиями и вычисляет, закрылся ли он.
   Дескриптор \a handler не должен разрушаться (\ref libfptr_destroy()), создаваться заново (\ref libfptr_create())
    и перенастраиваться (\ref libfptr_set_settings(), \ref libfptr_apply_single_settings()) между вызовами методов закрытия / формирования документа и \ref libfptr_check_document_closed()
    для корректной работы метода.

   Метод применим для:
   - чеков
   - отчетов закрытия и открытия смены
   - отчета регистрации, перерегистрации ККТ, закрытия архива ФН
   - отчета о состоянии расчетов

   Выходными свойствами метода являются \ref LIBFPTR_PARAM_DOCUMENT_CLOSED (закрылся документ или нет) и
   \ref LIBFPTR_PARAM_DOCUMENT_PRINTED (напечатался документ или нет).

   Если не обработать флаг \ref LIBFPTR_PARAM_DOCUMENT_PRINTED и не допечатать документа самостоятельно командой
    \ref libfptr_continue_print(), то документ будет автоматически допечатан при следующей печатной операции.

   Пример использования:
   \code{.cpp}
    libfptr_close_receipt(handle);
    do
    {
        if (libfptr_check_document_closed(handle) < 0)
        {
            switch (libfptr_error_code(handle))
            {
                case LIBFPTR_ERROR_PORT_NOT_AVAILABLE:
                case LIBFPTR_ERROR_PORT_BUSY:
                case LIBFPTR_ERROR_NO_CONNECTION:
                    // Не удалось установить связь с ККТ и проверить, закрылся ли документ
                    // (можно вывести сообщение пользователю, что нужно проверить соединение с ККТ)
                    break;
                default:
                    // Другие ошибки - провести проверку еще раз
                    break;
            }
            continue;
        }
    }
    while (true);

    bool documentClosed = (libfptr_get_param_bool(handle, LIBFPTR_PARAM_DOCUMENT_CLOSED) != 0);
    bool documentPrinted = (libfptr_get_param_bool(handle, LIBFPTR_PARAM_DOCUMENT_PRINTED) != 0);

    if (!documentClosed)
    {
        // Чек не закрылся - вернуть пользователю ошибку
        return;
    }

    if (!documentPrinted)
    {
        // Вывести пользователю сообщение, что документ недопечатан (можно также проверить, что
        // закрыта крышка, есть бумага и т.п.) и вызвать метод \ref libfptr_continue_print(),
        // когда пользователь решит, что проблема устранена
        while (libfptr_continue_print(handle) < 0)
        {
            // Вывести ошибку продолжения печати и повтять \ref libfptr_continue_print(),
            // пока документ не будет допечатан и метод не вернет 0.
        }
        break;
    }
    break;
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_check_document_closed(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Регистрирует итог чека

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_SUM     <td>Итог чека    <td>double <td>-
   </table>

   Команда не является обязательной. Если команда не передана, или \ref LIBFPTR_PARAM_SUM не указан, сумма чека будет
    посчитана автоматически. С помощью данной команды можно скорректировать сумму чека в рамках копеек.

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_receipt_total(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Регистрирует налог на чек.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_TAX_TYPE     <td>Ставка налога на чек    <td>int (\ref libfptr_tax_type)  <td>+
    <tr><td>\ref LIBFPTR_PARAM_TAX_SUM	    <td>Сумма налога на чек, р. <td>double                  <td>+
   </table>

   Указанная налоговая ставка должна присутствовать в чеке, чтобы регистрация ее суммы была возможна.

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_receipt_tax(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Производит регистрацию товара.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра  <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_COMMODITY_NAME       <td>Наименование товара                     <td>string                          <td>+
    <tr><td>\ref LIBFPTR_PARAM_PRICE                <td>Цена за единицу товара, р               <td>double                          <td>+
    <tr><td>\ref LIBFPTR_PARAM_QUANTITY             <td>Количество товара, ед.                  <td>double                          <td>+
    <tr><td>\ref LIBFPTR_PARAM_POSITION_SUM         <td>Общая сумма позиции, р.                 <td>double                          <td>- (автоматический рассчет)
    <tr><td>\ref LIBFPTR_PARAM_DEPARTMENT           <td>Номер отдела                            <td>int                             <td>1 (по возможности без печати на чековой ленте)
    <tr><td>\ref LIBFPTR_PARAM_TAX_TYPE             <td>Ставка налога на позицию                <td>int (\ref libfptr_tax_type)     <td>+
    <tr><td>\ref LIBFPTR_PARAM_TAX_SUM              <td>Сумма налога на позицию, р.             <td>double                          <td>- (сумма рассчитывается на стороне ККТ)
    <tr><td>\ref LIBFPTR_PARAM_USE_ONLY_TAX_TYPE    <td>Передавать только ставку налога         <td>bool                            <td>false
    <tr><td>\ref LIBFPTR_PARAM_TAX_MODE             <td>Способ начисления налога                <td>int (\ref libfptr_tax_mode)     <td>\ref LIBFPTR_TM_POSITION
    <tr><td>\ref LIBFPTR_PARAM_INFO_DISCOUNT_SUM    <td>Сумма информационной скидки/надбавки, р.<td>double                          <td>0.00 (не печатается на чековой ленте)
    <tr><td>\ref LIBFPTR_PARAM_COMMODITY_PIECE      <td>Штучный товар                           <td>bool                            <td>false
    <tr><td>\ref LIBFPTR_PARAM_CHECK_SUM            <td>Проверять сумму                         <td>bool                            <td>false

    <tr><th colspan="4">Допустимые реквизиты ФН
    <tr><td>1162                                    <td>Код товарной номенклатуры               <td>bytearray<td>
    <tr><td>1191                                    <td>Дополнительный реквизит товара          <td>string<td>
    <tr><td>1197                                    <td>Единицы измерения предмета расчета      <td>string<td>
    <tr><td>1212                                    <td>Признак предмета расчета                <td>int<td>
    <tr><td>1214                                    <td>Признак способа расчета                 <td>int<td>
    <tr><td>1222                                    <td>Признак агента по предмету расчета      <td>int<td>
    <tr><td>1223                                    <td>Данные агента                           <td>bytearray<td>
    <tr><td>1224                                    <td>Данные поставщика                       <td>bytearray<td>
    <tr><td>1226                                    <td>ИНН поставщика                          <td>string<td>
    <tr><td>1229                                    <td>Акциз                                   <td>double<td>
    <tr><td>1230                                    <td>Код страны производителя                <td>string<td>
    <tr><td>1231                                    <td>Номер таможенной декларации             <td>string<td>
   </table>

   <table>
    <caption>Описание реквизита 1223 (Данные агента)</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра
    <tr><td>1005        <td>Адрес оператора перевода                <td>string
    <tr><td>1016        <td>ИНН оператора перевода                  <td>string
    <tr><td>1026        <td>Наименование оператора перевода         <td>string
    <tr><td>1044        <td>Операция платежного агента              <td>string
    <tr><td>1073        <td>Телефон платежного агента               <td>string
    <tr><td>1074        <td>Телефон оператора по приему платежей    <td>string
    <tr><td>1075        <td>Телефон оператора перевода              <td>string
   </table>

   <table>
    <caption>Описание реквизита 1224 (Данные поставщика)</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра
    <tr><td>1171        <td>Телефон поставщика                      <td>string
    <tr><td>1225        <td>Наименование поставщика                 <td>string
   </table>

   Если \ref LIBFPTR_PARAM_POSITION_SUM не передан, то общая сумма расчитывается как \ref LIBFPTR_PARAM_PRICE *  \ref LIBFPTR_PARAM_QUANTITY с математическим округлением.<br>
   Если \ref LIBFPTR_PARAM_POSITION_SUM передан и \ref LIBFPTR_PARAM_POSITION_SUM != \ref LIBFPTR_PARAM_PRICE * \ref LIBFPTR_PARAM_QUANTITY с точностью до 1 копейки,
   производится распределение получившейся разницы (скидки/надбавки) на общее кол-во товаров в позиции.
   При этом возможно разбиение позиции на две, если невозможно распределить скидку/надбавку равномерно и ККТ реализует подобную логику.
   Сумма информационной скидки (\ref LIBFPTR_PARAM_INFO_DISCOUNT_SUM) в этом случае не учитывается.

   Пример:<br>
   цена за единицу = 100.00, количество = 7.000, общая сумма = 661.04.
   Скидка на позицию = 38.96, не распределяется равномерно на 7 единиц товара (38.96 = 5.56 * 3 + 5.57 * 4).
   Таким образом, на чековой ленте будет распечатано две позиции с одинаковым наименованием:
   1. 3.000 * 94.44 = 283.32
   2. 4.000 * 94.43 = 377.72

   Если \ref LIBFPTR_PARAM_TAX_SUM не передан, то ККТ рассчитает сумму налога самостоятельно.<br>
   Если \ref LIBFPTR_PARAM_TAX_SUM передан, то сумма налога будет зарегистрирована исходя из значения данного параметра.<br>
   Если \ref LIBFPTR_PARAM_USE_ONLY_TAX_TYPE передан и равен true, ККТ не будет рассчитывать сумму налога.
    В этом случае требуется зарегистрировать налог на чек с помощью \ref libfptr_receipt_tax().

   Для формирования составных реквизитов (1223, 1224) рекомендуется использовать \ref libfptr_util_form_tlv().
   Для упрощения формирования кода товарной номенклатуры (1162) можно использовать \ref libfptr_util_form_nomenclature().

   Пример использования:
   \code{.cpp}
    // Регистрация позиции без указания суммы налога (ККТ рассчитывает налоги самостоятельно)
    libfptr_set_param_str(handle, LIBFPTR_PARAM_COMMODITY_NAME, L"Товар");
    libfptr_set_param_double(handle, LIBFPTR_PARAM_PRICE, 100);
    libfptr_set_param_double(handle, LIBFPTR_PARAM_QUANTITY, 5.15);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_TAX_TYPE, LIBFPTR_TAX_VAT0);
    if (libfptr_registration(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Регистрация позиции с указанием суммы налога (ККТ проводит только логический контроль суммы)
    libfptr_set_param_str(handle, LIBFPTR_PARAM_COMMODITY_NAME, L"Товар");
    libfptr_set_param_double(handle, LIBFPTR_PARAM_PRICE, 100);
    libfptr_set_param_double(handle, LIBFPTR_PARAM_QUANTITY, 5.15);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_TAX_TYPE, LIBFPTR_TAX_VAT10);
    libfptr_set_param_double(handle, LIBFPTR_PARAM_TAX_SUM, 51.5);
    if (libfptr_registration(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Регистрация позиции без рассчета суммы налога (требуется регистрация суммы налогов в конце чека с помощью libfptr_tax())
    libfptr_set_param_str(handle, LIBFPTR_PARAM_COMMODITY_NAME, L"Товар");
    libfptr_set_param_double(handle, LIBFPTR_PARAM_PRICE, 100);
    libfptr_set_param_double(handle, LIBFPTR_PARAM_QUANTITY, 5.15);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_TAX_TYPE, LIBFPTR_TAX_VAT18);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_USE_ONLY_TAX_TYPE, true);
    if (libfptr_registration(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_registration(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Производит регистрацию оплаты.

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_PAYMENT_TYPE     <td>Тип оплаты                      <td>int (\ref libfptr_payment_type) <td>+
    <tr><td>\ref LIBFPTR_PARAM_PAYMENT_SUM      <td>Сумма оплаты, р.                <td>double <td>+
    <tr><th colspan="4">Выходные параметры
    <tr><td>\ref LIBFPTR_PARAM_REMAINDER        <td>Неоплаченный остаток чека, р.   <td colspan="2">double
    <tr><td>\ref LIBFPTR_PARAM_CHANGE           <td>Сдача, р.                       <td colspan="2">double
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_PAYMENT_TYPE, LIBFPTR_PT_CASH);
    libfptr_set_param_double(handle, LIBFPTR_PARAM_PAYMENT_SUM, 100.05);
    if (libfptr_payment(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    double remainder = libfptr_get_param_double(handle, LIBFPTR_PARAM_REMAINDER);
    double change = libfptr_get_param_double(handle, LIBFPTR_PARAM_CHANGE);
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_payment(libfptr_handle handle);

/*!
   \ingroup libfptr_group_report

   \brief Печатает отчет, заданный в параметре \ref LIBFPTR_PARAM_REPORT_TYPE

   Все входные параметры являются обязательными.

   <table>
    <caption>Список отчетов</caption>
    <tr><th rowspan="2">Тип данных (\ref LIBFPTR_PARAM_DATA_TYPE)       <th rowspan="2">Описание    <th colspan="3">Входные параметры
    <tr><th>Название<th>Описание<th>Тип параметра
    <tr><td>\ref LIBFPTR_RT_CLOSE_SHIFT                 <td>Закрытие смены<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_X                           <td>Суточный отчет без гашения<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_LAST_DOCUMENT               <td>Печать копии последнего чека<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_OFD_EXCHANGE_STATUS         <td>Состояние расчетов<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_KKT_DEMO                    <td>Демо-печать<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_KKT_INFO                    <td>Печать информации о ККТ<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_OFD_TEST                    <td>Диагностика соединения с ОФД<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_FN_DOC_BY_NUMBER            <td>Печать документа по номеру ФД из ФН<td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER <td>Номер документа<td>int
    <tr><td>\ref LIBFPTR_RT_FN_REGISTRATIONS            <td>Итоги регистраций/перерегистраций<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_QUANTITY                    <td>Отчет количеств<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_DEPARTMENTS                 <td>Отчет по секциям<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_OPERATORS                   <td>Отчет по кассирам<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_HOURS                       <td>Отчет по часам<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_FN_REGISTRATIONS            <td>Печать итогов регистрации / перерегистрации<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_FN_SHIFT_TOTAL_COUNTERS     <td>Печать счетчиков итогов смены<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_FN_TOTAL_COUNTERS           <td>Печать счетчиков итогов ФН<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_FN_NOT_SENT_DOCUMENTS_COUNTERS  <td>Печать счетчиков по непереданным документам<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_COMMODITIES_BY_TAXATION_TYPES <td>Печать отчета по товарам по СНО<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_COMMODITIES_BY_DEPARTMENTS  <td>Печать отчета по товарам по секциям<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_COMMODITIES_BY_SUMS         <td>Печать отчета по товарам по суммам<td colspan="3">
    <tr><td>\ref LIBFPTR_RT_START_SERVICE               <td>Начать служебный отчет<td colspan="3">
   </table>

   \warning В случае закрытия смены и отчета о состоянии расчетов
   требует проверки результата закрытия документа - \ref libfptr_check_document_closed().

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_REPORT_TYPE, LIBFPTR_RT_FN_DOC_BY_NUMBER);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_DOCUMENT_NUMBER, 10234);
    if (libfptr_report(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_report(libfptr_handle handle);

/*!
   \ingroup libfptr_group_text

   \brief Печатает текст на чековой ленте

   <table>
    <caption>Параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_TEXT                 <td>Текст                       <td>string  <td>Пустая строка
    <tr><td>\ref LIBFPTR_PARAM_ALIGNMENT            <td>Выравнивание текста         <td>int (\ref libfptr_alignment)    <td>\ref LIBFPTR_ALIGNMENT_LEFT
    <tr><td>\ref LIBFPTR_PARAM_TEXT_WRAP            <td>Перенос текста              <td>int (\ref libfptr_text_wrap)    <td>\ref LIBFPTR_TW_NONE
    <tr><td>\ref LIBFPTR_PARAM_FONT                 <td>Шрифт печати                <td>int     <td>По настройке ККТ
    <tr><td>\ref LIBFPTR_PARAM_FONT_DOUBLE_WIDTH    <td>Текст двойной ширины        <td>bool    <td>false
    <tr><td>\ref LIBFPTR_PARAM_FONT_DOUBLE_HEIGHT   <td>Текст двойной высоты        <td>bool    <td>false
    <tr><td>\ref LIBFPTR_PARAM_LINESPACING          <td>Межстрочный интервал        <td>int     <td>По настройке ККТ
    <tr><td>\ref LIBFPTR_PARAM_BRIGHTNESS           <td>Яркость текста              <td>int     <td>По настройке ККТ
   </table>

   \warning Рекомендуется печатать нефискальную информацию только внутри фискального или нефискального
    документов.

   Пример использования:
   \code{.cpp}
    libfptr_set_param_str(handle, LIBFPTR_PARAM_TEXT, L"Тестовая строка");
    libfptr_set_param_int(handle, LIBFPTR_PARAM_ALIGNMENT, LIBFPTR_ALIGNMENT_CENTER);
    if (libfptr_print_text(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_print_text(libfptr_handle handle);

/*!
   \ingroup libfptr_group_text

   \brief Печатает клише

   Пример использования:
   \code{.cpp}
    if (libfptr_print_cliche(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_print_cliche(libfptr_handle handle);

/*!
   \ingroup libfptr_group_text

   \brief Открывает нефискальный документ

   Пример использования:
   \code{.cpp}
    if (libfptr_begin_nonfiscal_document(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_begin_nonfiscal_document(libfptr_handle handle);

/*!
   \ingroup libfptr_group_text

   \brief Закрывает нефискальный документ

   Пример использования:
   \code{.cpp}
    if (libfptr_end_nonfiscal_document(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_end_nonfiscal_document(libfptr_handle handle);

/*!
   \ingroup libfptr_group_text

   \brief Печатает штрихкод

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_BARCODE                  <td>Данные ШК                   <td>string  <td>+
    <tr><td>\ref LIBFPTR_PARAM_BARCODE_TYPE             <td>Тип ШК                      <td>int (\ref libfptr_barcode_type) <td>+
    <tr><td>\ref LIBFPTR_PARAM_ALIGNMENT                <td>Выравнивание                <td>int (\ref libfptr_alignment) <td>\ref LIBFPTR_ALIGNMENT_CENTER
    <tr><td>\ref LIBFPTR_PARAM_SCALE                    <td>Коэффициент увеличения      <td>int <td>1
    <tr><td>\ref LIBFPTR_PARAM_LEFT_MARGIN              <td>Отступ слева, пикс.         <td>int <td>0
    <tr><td>\ref LIBFPTR_PARAM_BARCODE_PRINT_TEXT       <td>Флаг печати данных ШК       <td>bool    <td>false
    <tr><td>\ref LIBFPTR_PARAM_BARCODE_INVERT           <td>Инвертировать ШК            <td>bool    <td>false
   </table>

   <table>
    <caption>Дополнительные параметры по типам ШК</caption>
    <tr><th>Тип ШК    <th>Параметр      <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><td rowspan="1">\ref LIBFPTR_BT_EAN_8
        <br>\ref LIBFPTR_BT_EAN_13
        <br>\ref LIBFPTR_BT_UPC_A
        <br>\ref LIBFPTR_BT_UPC_E
        <br>\ref LIBFPTR_BT_CODE_39
        <br>\ref LIBFPTR_BT_CODE_93
        <br>\ref LIBFPTR_BT_CODE_128
        <br>\ref LIBFPTR_BT_CODABAR
        <br>\ref LIBFPTR_BT_ITF
        <br>\ref LIBFPTR_BT_ITF_14
        <br>\ref LIBFPTR_BT_GS1_128              <td>\ref LIBFPTR_PARAM_HEIGHT                          <td>Высота, пикс.                   <td>int <td>30
    <tr><td rowspan="2">\ref LIBFPTR_BT_QR       <td>\ref LIBFPTR_PARAM_BARCODE_CORRECTION              <td>Коррекция QR-кода               <td>int (\ref libfptr_barcode_correction) <td>\ref LIBFPTR_BC_DEFAULT
    <tr>                                         <td>\ref LIBFPTR_PARAM_BARCODE_VERSION                 <td>Версия QR-кода                  <td>int <td>Автоматически
    <tr><td rowspan="2">\ref LIBFPTR_BT_PDF417   <td>\ref LIBFPTR_PARAM_BARCODE_CORRECTION              <td>Коррекция PDF417                <td>int (\ref libfptr_barcode_correction) <td>\ref LIBFPTR_BC_DEFAULT
    <tr>                                         <td>\ref LIBFPTR_PARAM_BARCODE_COLUMNS                 <td>Количество столбцов PDF417      <td>int <td> 1
    <tr><td rowspan="2">\ref LIBFPTR_BT_AZTEC    <td>\ref LIBFPTR_PARAM_BARCODE_CORRECTION              <td>Коррекция Aztec                 <td>int (\ref libfptr_barcode_correction) <td>\ref LIBFPTR_BC_DEFAULT
    <tr>                                         <td>\ref LIBFPTR_PARAM_BARCODE_VERSION                 <td>Версия Aztec                    <td>int <td>Автоматически
   </table>

   Порядок применения параметров:
   1. \ref LIBFPTR_PARAM_HEIGHT
   2. \ref LIBFPTR_PARAM_SCALE
   3. \ref LIBFPTR_PARAM_ALIGNMENT
   4. \ref LIBFPTR_PARAM_LEFT_MARGIN

   Если ККТ не поддерживает печать ШК заданного типа, то библиотека сформирует ШК как изображение и попробует напечатать его.
   Допускается передача без контрольных символов (будут рассчитаны библиотекой).

   \warning Рекомендуется печатать нефискальную информацию только внутри фискального или нефискального
    документов.

   Пример использования:
   \code{.cpp}
    libfptr_set_param_str(handle, LIBFPTR_PARAM_BARCODE, L"123456789012");
    libfptr_set_param_int(handle, LIBFPTR_PARAM_BARCODE_TYPE, LIBFPTR_BT_EAN_13);
    if (libfptr_print_barcode(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_print_barcode(libfptr_handle handle);

/*!
   \ingroup libfptr_group_text

   \brief Печатает картинку из файла

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_FILENAME                 <td>Путь к файлу                <td>string <td>+
    <tr><td>\ref LIBFPTR_PARAM_ALIGNMENT                <td>Выравнивание                <td>int (\ref libfptr_alignment) <td>\ref LIBFPTR_ALIGNMENT_CENTER
    <tr><td>\ref LIBFPTR_PARAM_SCALE_PERCENT            <td>Масштаб, проценты           <td>int <td>100\%
    <tr><td>\ref LIBFPTR_PARAM_LEFT_MARGIN              <td>Отступ слева, пикс.         <td>int <td>0
   </table>

   Порядок применения параметров:
   1. \ref LIBFPTR_PARAM_SCALE_PERCENT
   2. \ref LIBFPTR_PARAM_ALIGNMENT
   3. \ref LIBFPTR_PARAM_LEFT_MARGIN

   Поддерживаются форматы:
   - bmp
   - png

   Изображение автоматически преобразуется в черно-белое.

   \warning Рекомендуется печатать нефискальную информацию только внутри фискального или нефискального
    документов.

   Пример использования:
   \code{.cpp}
    libfptr_set_param_str(handle, LIBFPTR_PARAM_FILENAME, L"С:\\picture.png");
    libfptr_set_param_int(handle, LIBFPTR_PARAM_ALIGNMENT, LIBFPTR_ALIGNMENT_LEFT);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_LEFT_MARGIN, 10);
    if (libfptr_print_picture(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_print_picture(libfptr_handle handle);

/*!
   \ingroup libfptr_group_text

   \brief Печатает картинку из памяти ККТ или внешнего хранилища.

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_PICTURE_NUMBER           <td>Номер картинки              <td>int <td>+
    <tr><td>\ref LIBFPTR_PARAM_ALIGNMENT                <td>Выравнивание                <td>int (\ref libfptr_alignment) <td>\ref LIBFPTR_ALIGNMENT_CENTER
    <tr><td>\ref LIBFPTR_PARAM_LEFT_MARGIN              <td>Отступ слева, пикс.         <td>int <td>0
   </table>

   Порядок применения параметров:
   1. \ref LIBFPTR_PARAM_ALIGNMENT
   2. \ref LIBFPTR_PARAM_LEFT_MARGIN

   \warning Рекомендуется печатать нефискальную информацию только внутри фискального или нефискального
    документов.

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_PICTURE_NUMBER, 1);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_ALIGNMENT, LIBFPTR_ALIGNMENT_CENTER);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_LEFT_MARGIN, 23);
    if (libfptr_print_picture_by_number(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_print_picture_by_number(libfptr_handle handle);

/*!
   \ingroup libfptr_group_programming

   \brief Загружает картинку в память ККТ или во внешнее хранилище.

   Метод подходит для загрузки статичных, часто используемых изображений, для ускорения их печати.
   Номер картинки, полученный в результате вызова метода, в дальнейшем может
    быть использован для печати картинки из памяти ККТ (\ref libfptr_print_picture_by_number).
   Некоторые ККТ позволяют печатать картинку, загруженную таким образом, в клише.

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_FILENAME                 <td>Путь к файлу                <td>string  <td>+
    <tr><td>\ref LIBFPTR_PARAM_SCALE_PERCENT            <td>Масштаб, проценты           <td>int     <td>100%
    <tr><th colspan="4">Выходные параметры
    <tr><td>\ref LIBFPTR_PARAM_PICTURE_NUMBER           <td>Номер картинки              <td colspan="2">int
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_str(handle, LIBFPTR_PARAM_FILENAME, L"С:\\picture.png");
    libfptr_set_param_double(handle, LIBFPTR_PARAM_SCALE_PERCENT, 50.0);
    if (libfptr_upload_picture_from_file(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    int pictureNum = libfptr_get_param_int(handle, LIBFPTR_PARAM_PICTURE_NUMBER);
   \endcode

   Поддерживаются форматы:
   - bmp
   - png


   Библиотека сама преобразует изображение в черно-белое.

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_upload_picture_from_file(libfptr_handle handle);

/*!
   \ingroup libfptr_group_programming

   \brief Очищает память ККТ или внешнее хранилище от картинок

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_clear_pictures(libfptr_handle handle);

/*!
   \ingroup libfptr_group_programming

   \brief Записывает настройку во внутреннюю таблицу ККТ.

   Возможные значения можно узнать в описании конкретной модели ККТ.

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_TABLE            <td>Номер таблицы       <td>int <td>+
    <tr><td>\ref LIBFPTR_PARAM_ROW              <td>Номер ряда          <td>int <td>+
    <tr><td>\ref LIBFPTR_PARAM_FIELD            <td>Номер поля          <td>int <td>+
    <tr><td>\ref LIBFPTR_PARAM_FIELD_VALUE      <td>Значение поля       <td>bytearray <td>+
   </table>

   Пример использования:
   \code{.cpp}
    uchar value[] = { 0x01 };
    libfptr_set_param_int(handle, LIBFPTR_PARAM_TABLE, 2);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_ROW, 1);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_FIELD, 1);
    libfptr_set_param_bytearray(handle, LIBFPTR_PARAM_FIELD_VALUE, value, sizeof(value) / sizeof(value[0]));
    if (libfptr_write_device_setting_raw(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_write_device_setting_raw(libfptr_handle handle);

/*!
   \ingroup libfptr_group_programming

   \brief Считывает настройку из внутренней таблицы ККТ.

   Возможные значения можно узнать в описании конкретной модели ККТ.

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_TABLE            <td>Номер таблицы       <td>int <td>+
    <tr><td>\ref LIBFPTR_PARAM_ROW              <td>Номер ряда          <td>int <td>+
    <tr><td>\ref LIBFPTR_PARAM_FIELD            <td>Номер поля          <td>int <td>+
    <tr><th colspan="4">Выходные параметры
    <tr><td>\ref LIBFPTR_PARAM_FIELD_VALUE      <td>Значение поля       <td colspan="2">bytearray
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_TABLE, 2);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_ROW, 1);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_FIELD, 1);
    if (libfptr_read_device_setting_raw(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    std::vector<uchar> value;
    int size = libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_FIELD_VALUE, &value[0], value.size());
    if (size > value.size())
    {
        value.resize(size);
        libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_FIELD_VALUE, &value[0], value.size());
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_read_device_setting_raw(libfptr_handle handle);

/*!
   \ingroup libfptr_group_programming

   \brief Применяет записанные методами \ref libfptr_write_device_setting_raw() и \ref libfptr_write_device_setting() настройки.

   Поведение метода зависит от ККТ, но требуется всегда вызывать его после программирования набора настроек ККТ, иначе
    ККТ может восстановить старые настройки при перезагрузке.

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_commit_settings(libfptr_handle handle);

/*!
   \ingroup libfptr_group_programming

   \brief Инициализирует системные таблицы начальными значениями

   \warning Для корректной инициализации таблиц необходимо после вызова метода выключить и
   включить ККТ.

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_init_settings(libfptr_handle handle);

/*!
   \ingroup libfptr_group_service

   \brief Производит технологическое обнуление ККТ

   Метод производит технологическое обнуление ККТ. Технологическое обнуление включает в себя:
   - инициализацию системных таблиц начальными значениями;
   - общее гашение ККТ;
   - обнуление счетчика общих гашений.

   Подробности использования технологического обнуления находятся в руководстве на ККТ.

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_reset_settings(libfptr_handle handle);

/*!
   \ingroup libfptr_group_service

   \brief Устанавливает дату и время в ККТ

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_DATE_TIME        <td>Дата и время        <td>datetime    <td>Текущие дата и время
   </table>

   Пример использования:
   \code{.cpp}
    // Дата и время - 02.01.2017 12:13:45
    libfptr_set_param_datetime(handle, LIBFPTR_PARAM_DATE_TIME, 2017, 1, 2, 12, 13, 45);
    if (libfptr_write_date_time(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_write_date_time(libfptr_handle handle);

/*!
   \ingroup libfptr_group_service

   \brief Записывает код защиты в ККТ

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_LICENSE_NUMBER   <td>Номер кода защиты       <td>int     <td>+
    <tr><td>\ref LIBFPTR_PARAM_LICENSE          <td>Значение кода защиты    <td>string  <td>+
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_LICENSE_NUMBER, 4);
    libfptr_set_param_str(handle, LIBFPTR_PARAM_LICENSE, L"1234567890");
    if (libfptr_write_license(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_write_license(libfptr_handle handle);

/*!
   \ingroup libfptr_group_fn

   \brief Выполняет операцию ФН

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание                        <th>Тип параметра
    <tr><th colspan="3">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_FN_OPERATION_TYPE
        <td>Тип операции ФН
        <td>int (\ref libfptr_fn_operation_type)
    <tr><th colspan="3">Реквизиты для регистрации/перегистрации ККТ
    <tr><td>1001        <td>Автоматический режим            <td>bool
    <tr><td>1002        <td>Автономный режим                <td>bool
    <tr><td>1009        <td>Адрес (место) расчетов          <td>string
    <tr><td>1017        <td>ИНН ОФД                         <td>string
    <tr><td>1018        <td>ИНН пользователя                <td>string
    <tr><td>1036        <td>Номер автомата                  <td>string
    <tr><td>1037        <td>Регистрационный номер ККТ       <td>string
    <tr><td>1046        <td>Наименование ОФД                <td>string
    <tr><td>1048        <td>Наименование пользователя       <td>string
    <tr><td>1056        <td>Признак шифрования              <td>bool
    <tr><td>1057        <td>Признак платежного агента       <td>int
    <tr><td>1060        <td>Адрес сайта ФНС                 <td>string
    <tr><td>1062        <td>Системы налогообложения         <td>int
    <tr><td>1101        <td>Код причины перерегистрации     <td>int
    <tr><td>1108        <td>Признак расчетов в Интернете    <td>bool
    <tr><td>1110        <td>Признак АС БСО                  <td>bool
    <tr><td>1117        <td>E-mail организации              <td>string
    <tr><td>1109        <td>Признак услуги                  <td>bool
    <tr><td>1126        <td>Признак проведения лотереи      <td>bool
    <tr><td>1187        <td>Место расчетов                  <td>string
    <tr><td>1193        <td>Признак проведения азартных игр <td>bool
    <tr><td>1205        <td>Причина измененения параметров ККТ <td>int
    <tr><td>1207        <td>Признак подакцизного товара     <td>bool
    <tr><td>1209        <td>Версия ФФД                      <td>int (\ref libfptr_ffd_version)
    <tr><td>1221        <td>Признак установки принтера в автомате   <td>bool
   </table>

   \warning Требует проверки результата закрытия документа - \ref libfptr_check_document_closed().

   Пример использования:
   \code{.cpp}
    libfptr_set_param_bool(handle, LIBFPTR_PARAM_FN_OPERATION_TYPE, LIBFPTR_FNOP_CHANGE_PARAMETERS);
    libfptr_set_param_bool(handle, 1001, false);
    libfptr_set_param_bool(handle, 1002, false);
    libfptr_set_param_bool(handle, 1056, false);
    libfptr_set_param_bool(handle, 1108, true);
    libfptr_set_param_bool(handle, 1109, false);
    libfptr_set_param_bool(handle, 1221, false);
    libfptr_set_param_int(handle, 1057, 0x01 | 0x04 | 0x10);
    libfptr_set_param_int(handle, 1062, 0x01 | 0x20);
    libfptr_set_param_int(handle, 1101, 4);
    libfptr_set_param_str(handle, 1009, L"Улица свободы, 42");
    libfptr_set_param_str(handle, 1017, L"123456789012");
    libfptr_set_param_str(handle, 1018, L"135792468001");
    libfptr_set_param_str(handle, 1036, L"513");
    libfptr_set_param_str(handle, 1037, L"12345678900987654321");
    libfptr_set_param_str(handle, 1046, L"ОФД \"ОФД\"");
    libfptr_set_param_str(handle, 1048, L"Иванов Иван Иванович");
    libfptr_set_param_str(handle, 1187, L"Улица свободы, 42");
    libfptr_set_param_int(handle, 1209, LIBFPTR_FFD_1_0);
    if (libfptr_fn_operation(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_fn_operation(libfptr_handle handle);


/*!
   \ingroup libfptr_group_fn

   \brief Запрашивает информацию из фискального накопителя

   Все входные параметры являются обязательными.

   <table>
    <caption>Общие параметры</caption>
    <tr><th rowspan="2">Тип данных (LIBFPTR_PARAM_FN_DATA_TYPE)       <th rowspan="2">Описание    <th colspan="3">Входные параметры   <th colspan="3">Выходные параметры
    <tr><th>Название<th>Описание<th>Тип параметра<th>Название<th>Описание<th>Тип параметра

    <tr><td rowspan="3">\ref LIBFPTR_FNDT_TAG_VALUE                 <td rowspan="3">Значение реквизита регистрации
    <tr><td rowspan="2">\ref LIBFPTR_PARAM_TAG_NUMBER                   <td rowspan="2">Номер реквизита  <td rowspan="2">int
    <tr>            <td>\ref LIBFPTR_PARAM_TAG_VALUE                    <td>Значение реквизита           <td>Зависит от номера реквизита (см. ниже описание \ref LIBFPTR_FNDT_REG_INFO)

    <tr><td rowspan="7">\ref LIBFPTR_FNDT_OFD_EXCHANGE_STATUS           <td rowspan="7">Статус информационного обмена
    <tr><td rowspan="6" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_OFD_EXCHANGE_STATUS          <td>Статус информационного обмена   <td>int
                    <br>Битовое поле:
                    <br>Бит 0 – транспортное соединение установлено
                    <br>Бит 1 – есть сообщение для передачи в ОФД
                    <br>Бит 2 – ожидание ответного сообщения (квитанции) от ОФД
                    <br>Бит 3 – есть команда от ОФД
                    <br>Бит 4 – изменились настройки соединения с ОФД
                    <br>Бит 5 – ожидание ответа на команду от ОФД
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENTS_COUNT              <td>Количество непереданных документов              <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER              <td>Номер первого непереданного документа           <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_DATE_TIME                    <td>Дата и время первого непереданного документа    <td>datetime
    <tr>            <td>\ref LIBFPTR_PARAM_OFD_MESSAGE_READ             <td>Производится чтение сообщения для ОФД           <td>bool

    <tr><td rowspan="11">\ref LIBFPTR_FNDT_FN_INFO                   <td rowspan="11">Информация об ФН
    <tr><td rowspan="10" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_SERIAL_NUMBER                <td>Серийный номер          <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_FN_VERSION                   <td>Версия                  <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_FN_TYPE                      <td>Тип ФН                  <td>int (\ref libfptr_fn_type)
    <tr>            <td>\ref LIBFPTR_PARAM_FN_STATE                     <td>Состояние ФН            <td>int (\ref libfptr_fn_state)
    <tr>            <td>\ref LIBFPTR_PARAM_FN_NEED_REPLACEMENT          <td>Требуется срочная замена ФН <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_FN_RESOURCE_EXHAUSTED        <td>Исчерпан ресурс ФН      <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_FN_MEMORY_OVERFLOW           <td>Память ФН переполнена   <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_FN_OFD_TIMEOUT               <td>Превышено время ожидания ответа от ОФД  <td>bool
    <tr>            <td>\ref LIBFPTR_PARAM_FN_CRITICAL_ERROR            <td>Критическая ошибка ФН   <td>bool

    <tr><td rowspan="5">\ref LIBFPTR_FNDT_LAST_REGISTRATION         <td rowspan="5">Информация о последней регистрации/перерегистрации
    <tr><td rowspan="4" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_DATE_TIME                    <td>Дата и время                <td>datetime
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER              <td>Номер фискального документа <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_REGISTRATIONS_COUNT          <td>Номер перерегистрации       <td>int

    <tr><td rowspan="7">\ref LIBFPTR_FNDT_LAST_RECEIPT              <td rowspan="7">Информация о последнем чеке
    <tr><td rowspan="6" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER              <td>Номер фискального документа чека    <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_FISCAL_SIGN                  <td>Фискальный признак документа чека   <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_SUM                  <td>Сумма, р.                           <td>double
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_TYPE                 <td>Тип чека                            <td>int (\ref libfptr_receipt_type)
    <tr>            <td>\ref LIBFPTR_PARAM_DATE_TIME                    <td>Дата и время                        <td>datetime

    <tr><td rowspan="5">\ref LIBFPTR_FNDT_LAST_DOCUMENT             <td rowspan="5">Информация о последнем документе
    <tr><td rowspan="4" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER              <td>Номер фискального документа <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_FISCAL_SIGN                  <td>Фискальный признак документа<td>string
    <tr>            <td>\ref LIBFPTR_PARAM_DATE_TIME                    <td>Дата и время                <td>datetime

    <tr><td rowspan="4">\ref LIBFPTR_FNDT_SHIFT                     <td rowspan="4">Информация о смене
    <tr><td rowspan="3" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_RECEIPT_NUMBER               <td>Кол-во чеков за смену (номер последнего чека)   <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_SHIFT_NUMBER                 <td>Номер смены                                     <td>int

    <tr><td rowspan="7">\ref LIBFPTR_FNDT_FFD_VERSIONS              <td rowspan="7">Информация о версиях ФФД
    <tr><td rowspan="6" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_DEVICE_FFD_VERSION           <td>Версия ФФД ККТ          <td>int (\ref libfptr_ffd_version)
    <tr>            <td>\ref LIBFPTR_PARAM_FN_FFD_VERSION               <td>Версия ФФД ФН           <td>int (\ref libfptr_ffd_version)
    <tr>            <td>\ref LIBFPTR_PARAM_FFD_VERSION                  <td>Версия ФФД              <td>int (\ref libfptr_ffd_version)
    <tr>            <td>\ref LIBFPTR_PARAM_DEVICE_MIN_FFD_VERSION       <td>Минимальная версия ФФД ККТ  <td>int (\ref libfptr_ffd_version)
    <tr>            <td>\ref LIBFPTR_PARAM_DEVICE_MAX_FFD_VERSION       <td>Максимальная версия ФФД ККТ <td>int (\ref libfptr_ffd_version)

    <tr><td rowspan="5">\ref LIBFPTR_FNDT_VALIDITY                  <td rowspan="5">Срок действия
    <tr><td rowspan="4" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_DATE_TIME                    <td>Срок действия               <td>datetime
    <tr>            <td>\ref LIBFPTR_PARAM_REGISTRATIONS_REMAIN         <td>Осталось перерегистраций    <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_REGISTRATIONS_COUNT          <td>Сделано перерегистраций     <td>int

    <tr><td rowspan="25">\ref LIBFPTR_FNDT_REG_INFO                 <td rowspan="25">Регистрационные данные
    <tr><td rowspan="24" colspan="3">
    <tr>            <td>1001                                        <td>Признак автоматического режима  <td>bool
    <tr>            <td>1002                                        <td>Признак автономного режима  <td>bool
    <tr>            <td>1009                                        <td>Адрес расчетов              <td>string
    <tr>            <td>1017                                        <td>ИНН ОФД                     <td>string
    <tr>            <td>1018                                        <td>ИНН организации             <td>string
    <tr>            <td>1036                                        <td>Номер автомата              <td>string
    <tr>            <td>1037                                        <td>Регистрационный номер ККТ   <td>string
    <tr>            <td>1046                                        <td>Наименование ОФД            <td>string
    <tr>            <td>1048                                        <td>Название организации        <td>string
    <tr>            <td>1056                                        <td>Признак шифрования          <td>bool
    <tr>            <td>1057                                        <td>Признак агента              <td>int
    <tr>            <td>1060                                        <td>Адрес сайта ФНС             <td>string
    <tr>            <td>1062                                        <td>Системы налогообложения     <td>int
    <tr>            <td>1108                                        <td>Признак ККТ для расчетов только в Интернет  <td>bool
    <tr>            <td>1109                                        <td>Признак расчетов за услуги  <td>bool
    <tr>            <td>1110                                        <td>Признак АС БСО              <td>bool
                                                                                                    <br>Битовое поле, соответствует \ref libfptr_taxation_type
    <tr>            <td>1117                                        <td>E-mail организации          <td>string
    <tr>            <td>1126                                        <td>Признак проведения лотереи  <td>bool
    <tr>            <td>1187                                        <td>Место расчетов              <td>string
    <tr>            <td>1193                                        <td>Признак проведения азартных игр  <td>bool
    <tr>            <td>1207                                        <td>Признак подакцизного товара <td>bool
    <tr>            <td>1209                                        <td>Версия ФФД                  <td>int (\ref libfptr_ffd_version)
    <tr>            <td>1221                                        <td>Признак установки принтера в автомате  <td>bool

    <tr><td>\ref LIBFPTR_FNDT_DOCUMENTS_COUNT_IN_SHIFT  <td>Количество ФД за смену (значение имеет смысл только после закрытия смены)
        <td colspan="3"><td>\ref LIBFPTR_PARAM_DOCUMENTS_COUNT          <td>Количество ФД            <td>int

    <tr><td rowspan="10">\ref LIBFPTR_FNDT_ERRORS                    <td rowspan="10">Ошибки ФН / ОФД / сети
    <tr><td rowspan="9" colspan="3">
    <tr>            <td>\ref LIBFPTR_PARAM_NETWORK_ERROR            <td>Код ошибки сети         <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_NETWORK_ERROR_TEXT       <td>Описание ошибки сети    <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_OFD_ERROR                <td>Код ошибки ОФД          <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_OFD_ERROR_TEXT           <td>Описание ошибки ОФД     <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_FN_ERROR                 <td>Код ошибки ФН           <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_FN_ERROR_TEXT            <td>Описание ошибки ФН      <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER          <td>Номер ФД, на котором произошла ошибка       <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_COMMAND_CODE             <td>Код команды ФН, на которой произошла ошибка <td>int

    <tr><td rowspan="4">\ref LIBFPTR_FNDT_TICKET_BY_DOC_NUMBER      <td rowspan="4">Квитанция ОФД по номеру документа
    <tr><td rowspan="3">\ref LIBFPTR_PARAM_DOCUMENT_NUMBER          <td rowspan="3">Номер ФД  <td rowspan="3">int
                    <td>\ref LIBFPTR_PARAM_DATE_TIME                <td>Дата и время квитанции из ОФД (UTC) <td>datetime
    <tr>            <td>\ref LIBFPTR_PARAM_OFD_FISCAL_SIGN          <td>Фискальный признак ОФД <td>bytearray
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER          <td>Номер ФД <td>int

    <tr><td rowspan="5">\ref LIBFPTR_FNDT_DOCUMENT_BY_NUMBER        <td rowspan="5">Информация о документе по номеру
    <tr><td rowspan="4">\ref LIBFPTR_PARAM_DOCUMENT_NUMBER          <td rowspan="4">Номер ФД  <td rowspan="4">int
                    <td>\ref LIBFPTR_PARAM_DATE_TIME                <td>Дата и время ФД       <td>datetime
    <tr>            <td>\ref LIBFPTR_PARAM_DOCUMENT_NUMBER          <td>Номер ФД              <td>int
    <tr>            <td>\ref LIBFPTR_PARAM_FISCAL_SIGN              <td>Фискальный признак документа чека   <td>string
    <tr>            <td>\ref LIBFPTR_PARAM_HAS_OFD_TICKET           <td>Флаг подтверждения от ОФД   <td>bool

   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_FN_DATA_TYPE, LIBFPTR_FNDT_DOCUMENT_BY_NUMBER);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_DOCUMENT_NUMBER, 12353);
    if (libfptr_fn_query_data(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Получение результатов запроса
    bool hasTicket = (libfptr_get_param_bool(handle, LIBFPTR_PARAM_HAS_OFD_TICKET) != 0);
    // ...
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_fn_query_data(libfptr_handle handle);

/*!
   \ingroup libfptr_group_fn

   \brief Записывает набор реквизитов в ККТ

   Входными параметрами метода является набор произвольных реквизитов,
    которые будут переданы в ККТ.

   Пример использования:
   \code{.cpp}
    // Адрес сайта ФНС
    libfptr_set_param_str(handle, 1060, L"www.nalog.ru");
    // Адрес электронной почты отправителя чека
    libfptr_set_param_str(handle, 1075, L"sender@company.ru");
    if (libfptr_fn_write_attributes(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_fn_write_attributes(libfptr_handle handle);

/*!
   \ingroup libfptr_group_external_dev

   \todo Сделать

   \brief Активирует внешнее устройство

   Метод используется для открытия соединения с внешним устройством (и подачи питания на него, если требуется).
   После вызова данного метода можно обмениваться с ним с помощью \ref libfptr_external_device_write_data() и \ref libfptr_external_device_read_data().
   Для отлючения устройства и прекращения обмена с ним необходимо вызвать \ref libfptr_external_device_power_off().

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE   <td>Тип устройства    <td>int (\ref libfptr_external_device_type) <td>+
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE, LIBFPTR_EXTERNAL_DEVICE_PINPAD);
    if (libfptr_external_device_power_on(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_external_device_power_on(libfptr_handle handle);

/*!
   \ingroup libfptr_group_external_dev

   \todo Сделать

   \brief Деактивирует внешнее устройство

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE   <td>Тип устройства    <td>int (\ref libfptr_external_device_type) <td>+
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE, LIBFPTR_EXTERNAL_DEVICE_PINPAD);
    if (libfptr_external_device_power_off(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_external_device_power_off(libfptr_handle handle);

/*!
   \ingroup libfptr_group_external_dev

   \todo Сделать

   \brief Отправляет внешнему устройству набор байтов

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE   <td>Тип устройства        <td>int (\ref libfptr_external_device_type) <td>+
    <tr><td>\ref LIBFPTR_PARAM_EXTERNAL_DEVICE_DATA   <td>Данные для устройства <td>bytearray   <td>+
   </table>

   Пример использования:
   \code{.cpp}
    uchar data[] = {0x01, 0x02, 0x03};
    libfptr_set_param_int(handle, LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE, LIBFPTR_EXTERNAL_DEVICE_PINPAD);
    libfptr_set_param_bytearray(handle, LIBFPTR_PARAM_EXTERNAL_DEVICE_DATA, &data[0], sizeof(data) / sizeof(data[0]));
    if (libfptr_external_device_write_data(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_external_device_write_data(libfptr_handle handle);

/*!
   \ingroup libfptr_group_external_dev

   \todo Сделать

   \brief Возвращает набор байтов от внешнего устройства

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE   <td>Тип устройства        <td>int (\ref libfptr_external_device_type) <td>+
    <tr><th colspan="4">Выходные параметры
    <tr><td>\ref LIBFPTR_PARAM_EXTERNAL_DEVICE_DATA   <td>Данные от устройства <td colspan="2">bytearray
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_EXTERNAL_DEVICE_TYPE, LIBFPTR_EXTERNAL_DEVICE_PINPAD);
    if (libfptr_external_device_write_data(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    std::vector<uchar> data;
    int size = libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_EXTERNAL_DEVICE_DATA, &data[0], data.size());
    if (size > data.size())
    {
        data.resize(size);
        libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_EXTERNAL_DEVICE_DATA, &data[0], data.size());
    }

    // Обработка данных data
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_external_device_read_data(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Регистрирует кассира

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>1021    <td>Должность и фамилия кассира         <td>string <td>
    <tr><td>1203    <td>ИНН кассира                         <td>string <td>
   </table>

   \warning Рекомендуется вызывать данный метод перед каждой фискальной операцией (формирование чека, снятие отчета).

   \param handle
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_operator_login(libfptr_handle handle);

/*!
   \ingroup libfptr_group_json

   \brief Выполняет задание, указанное в JSON (см. \ref libfptr_page_json)

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_JSON_DATA    <td>JSON-задание             <td>string <td>+
    <tr><th colspan="4">Выходные параметры
    <tr><td>\ref LIBFPTR_PARAM_JSON_DATA    <td>Результат JSON-задания   <td>string <td>+
   </table>

   \param handle
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_process_json(libfptr_handle handle);

/*!
   \ingroup libfptr_group_programming

   \brief Считывает настройку из внутренней таблицы ККТ.

   Данный метод предназначен для унификации работы с настройками ККТ. Каждому из параметров (\ref libfptr_kkt_parameters)
    соответствует настройка во внутренних таблицах ККТ. Возвращаемые из метода значения регламентированы библиотекой и
    будут автоматически преобразованы из значений, соответствующих текущей модели ККТ.

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_SETTING_ID       <td>Номер настройки     <td>int <td>+
    <tr><th colspan="4">Выходные параметры
    <tr><td>\ref LIBFPTR_PARAM_SETTING_VALUE    <td>Значение настройки  <td colspan="2">Зависит от настройки
   </table>

   Пример использования:
   \code{.cpp}
    // Чтение настройки "DNS ОФД"
    libfptr_set_param_int(handle, LIBFPTR_PARAM_SETTING_ID, 296);
    libfptr_set_param_str(handle, LIBFPTR_PARAM_SETTING_VALUE, L"8.8.8.8");
    if (libfptr_read_device_setting(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
    std::vector<wchar_t> answerStr(1024);
    int size = libfptr_get_param_str(handle, LIBFPTR_PARAM_SETTING_VALUE, &answerStr[0], answerStr.size());
    if (size > answerStr.length())
    {
        answerStr.resize(size);
        libfptr_get_param_str(handle, LIBFPTR_PARAM_ANSWER_BUFFER, &answerStr[0], answerStr.size());
    }


    // Чтение настройки "Порт ОФД"
    libfptr_set_param_int(handle, LIBFPTR_PARAM_SETTING_ID, 295);
    if (libfptr_read_device_setting(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
    int answerInt = libfptr_get_param_int(handle, LIBFPTR_PARAM_SETTING_VALUE);
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_read_device_setting(libfptr_handle handle);

/*!
   \ingroup libfptr_group_programming

   \brief Записывает настройку во внутреннюю таблицу ККТ.

   Данный метод предназначен для унификации работы с настройками ККТ. Каждому из параметров (\ref libfptr_kkt_parameters)
    соответствует настройка во внутренних таблицах ККТ. Передаваемые в метод значения регламентированы библиотекой и
    будут автоматически преобразованы в подходящие для текущей ККТ значения.

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_SETTING_ID       <td>Номер настройки     <td>int <td>+
    <tr><td>\ref LIBFPTR_PARAM_SETTING_VALUE    <td>Значение настройки  <td>Зависит от настройки <td>+
   </table>

   Пример использования:
   \code{.cpp}
    // Запись настройки "DNS ОФД"
    libfptr_set_param_int(handle, LIBFPTR_PARAM_SETTING_ID, 296);
    libfptr_set_param_str(handle, LIBFPTR_PARAM_SETTING_VALUE, L"8.8.8.8");
    if (libfptr_write_device_setting(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    // Запись настройки "Порт ОФД"
    libfptr_set_param_int(handle, LIBFPTR_PARAM_SETTING_ID, 295);
    libfptr_set_param_int(handle, LIBFPTR_PARAM_SETTING_VALUE, 19801);
    if (libfptr_write_device_setting(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_write_device_setting(libfptr_handle handle);

/*!
   \ingroup libfptr_group_read_data

   \brief Начинает процесс чтения данных.

   Запускает процесс чтения данных с идентификатором \ref LIBFPTR_PARAM_RECORDS_TYPE.
   Данные кешируются в библиотеке, дальнейшие вызовы \ref libfptr_read_next_record()
    считывают их из памяти библиотеки.
   Данный метод также может вернуть набор данных, общий для всех записей.

   <table>
    <caption>Общие параметры</caption>
    <tr><th colspan="2">Тип данных (LIBFPTR_PARAM_RECORDS_TYPE)    <th colspan="3">Входные параметры <th colspan="3">Выходные параметры
    <tr><th>Название<th>Описание                                        <th>Название<th>Описание<th>Тип параметра<th>Название<th>Описание<th>Тип параметра
    <tr><td>\ref LIBFPTR_RT_LAST_DOCUMENT_LINES <td>Строки последнего напечатанного документа
                        <td colspan="3">
                        <td colspan="3">

    <tr><td rowspan="2">\ref LIBFPTR_RT_FN_DOCUMENT_TLVS    <td rowspan="2">Реквизиты документа по номеру из ФН в виде TLV
                        <td rowspan="2">\ref LIBFPTR_PARAM_DOCUMENT_NUMBER <td rowspan="2">Номер ФД <td rowspan="2">int
                        <td>\ref LIBFPTR_PARAM_FN_DOCUMENT_TYPE <td>Тип документа <td>int (\ref libfptr_fn_document_type)
    <tr>                <td>\ref LIBFPTR_PARAM_COUNT            <td>Размер документа <td>int

    <tr><td rowspan="2">\ref LIBFPTR_RT_EXEC_USER_SCRIPT    <td rowspan="2">Выполнение скрипта пользователя и получение его результатов
                        <td>\ref LIBFPTR_PARAM_USER_SCRIPT_ID <td>Идентификатор скрипта <td>string <td colspan="3" rowspan="2">
    <tr><td>\ref LIBFPTR_PARAM_USER_SCRIPT_PARAMETER <td>Параметр скрипта <td>bytearray


   </table>

   Общий алгоритм работы:
   \code{.cpp}
    libfptr_set_param_int(handle, LIBFPTR_PARAM_RECORDS_TYPE, LIBFPTR_RT_LAST_DOCUMENT_LINES);
    if (libfptr_begin_read_records(handle) < 0)
    {
        // Обработка ошибки - данные получить не удалось
    }

    // Прочитать общие параметры
    libfptr_get_param_str(handle, ...);

    while (libfptr_read_next_record() == LIBFPTR_OK)
    {
        // Получить и обработать информацию о записи
        libfptr_get_param_int(handle, ...);
        libfptr_get_param_str(handle, ...);
    }
    libfptr_end_read_records(handle);
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_begin_read_records(libfptr_handle handle);

/*!
   \ingroup libfptr_group_read_data

   \brief Считывает запись данных

   Считывает запись данных из ККТ в параметры библиотеки. Набор параметров зависит от типа данных
    (\ref LIBFPTR_PARAM_RECORDS_TYPE).

   <table>
    <caption>Общие параметры</caption>
    <tr><th colspan="2">Тип данных (LIBFPTR_PARAM_RECORDS_TYPE)    <th colspan="3">Выходные параметры каждого элемента данных
    <tr><th>Название<th>Описание                                        <th>Название<th>Описание<th>Тип параметра
    <tr><td rowspan="6">\ref LIBFPTR_RT_LAST_DOCUMENT_LINES
        <td rowspan="6">Строки последнего напечатанного документа   <td>\ref LIBFPTR_PARAM_TEXT                 <td>Текст                       <td>string
    <tr>                                                            <td>\ref LIBFPTR_PARAM_FONT                 <td>Шрифт печати                <td>int
    <tr>                                                            <td>\ref LIBFPTR_PARAM_FONT_DOUBLE_WIDTH    <td>Текст двойной ширины        <td>bool
    <tr>                                                            <td>\ref LIBFPTR_PARAM_FONT_DOUBLE_HEIGHT   <td>Текст двойной высоты        <td>bool
    <tr>                                                            <td>\ref LIBFPTR_PARAM_LINESPACING          <td>Межстрочный интервал        <td>int
    <tr>                                                            <td>\ref LIBFPTR_PARAM_BRIGHTNESS           <td>Яркость текста              <td>int

    <tr><td rowspan="2">\ref LIBFPTR_RT_FN_DOCUMENT_TLVS
        <td rowspan="2">Реквизиты документа по номеру из ФН в виде TLV.
                    <br>Каждый вызов \ref libfptr_read_next_record() читает очередную TLV верхнего уровня.
        <td>\ref LIBFPTR_PARAM_TAG_NUMBER <td>Номер реквизита ФН <td>int
    <tr><td>\ref LIBFPTR_PARAM_TAG_VALUE  <td>Данные реквизита ФН <td>bytearray

    <tr><td>\ref LIBFPTR_RT_EXEC_USER_SCRIPT
        <td>Выполнение скрипта пользователя и получение его результатов
        <td>\ref LIBFPTR_PARAM_USER_SCRIPT_PARAMETER <td>Выходной параметр скрипта <td>bytearray

   </table>

   Общий алгоритм работы приведен в описании метода \ref libfptr_begin_read_records().

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_read_next_record(libfptr_handle handle);

/*!
   \ingroup libfptr_group_read_data

   \brief Завершает процесс чтения данных.

   Общий алгоритм работы приведен в описании метода \ref libfptr_begin_read_records().

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_end_read_records(libfptr_handle handle);

/*!
   \ingroup libfptr_group_user_memory

   \brief Производит операцию с пользовательской памятью ККТ.

   Операция задается в параметре \ref LIBFPTR_PARAM_USER_MEMORY_OPERATION.

   <table>
    <caption>Общие параметры</caption>
    <tr><th colspan="2">Операция (LIBFPTR_PARAM_USER_MEMORY_OPERATION)  <th colspan="3">Входные параметры <th colspan="3">Выходные параметры
    <tr><th>Название<th>Описание <th>Название<th>Описание<th>Тип параметра <th>Название<th>Описание<th>Тип параметра
    <tr><td>\ref LIBFPTR_UMO_GET_SIZE  <td>Запрос размера памяти
        <td colspan="3">
        <td>\ref LIBFPTR_PARAM_COUNT <td>Количество ячеек памяти <td>int

    <tr><td rowspan="2">\ref LIBFPTR_UMO_READ_DATA  <td rowspan="2">Чтение данных из памяти
        <td>\ref LIBFPTR_PARAM_USER_MEMORY_ADDRESS <td>Адрес в памяти <td>int
                                        <td rowspan="2">\ref LIBFPTR_PARAM_USER_MEMORY_DATA <td rowspan="2">Данные <td rowspan="2">bytearray
    <tr><td>\ref LIBFPTR_PARAM_COUNT <td>Количество ячеек памяти для чтения <td>int

    <tr><td rowspan="2">\ref LIBFPTR_UMO_WRITE_DATA  <td rowspan="2">Запись данных в память
        <td>\ref LIBFPTR_PARAM_USER_MEMORY_ADDRESS <td>Адрес в памяти <td>int
                                        <td rowspan="2" colspan="3">
    <tr><td>\ref LIBFPTR_PARAM_USER_MEMORY_DATA <td>Данные <td>bytearray

    <tr><td>\ref LIBFPTR_UMO_READ_STRING  <td>Чтение строки из памяти
        <td>\ref LIBFPTR_PARAM_USER_MEMORY_ADDRESS <td>Адрес в памяти <td>int <td>\ref LIBFPTR_PARAM_USER_MEMORY_STRING <td>Строка <td>string

    <tr><td rowspan="2">\ref LIBFPTR_UMO_WRITE_STRING  <td rowspan="2">Запись строки в память
        <td>\ref LIBFPTR_PARAM_USER_MEMORY_ADDRESS <td>Адрес в памяти <td>int
                                        <td rowspan="2" colspan="3">
    <tr><td>\ref LIBFPTR_PARAM_USER_MEMORY_STRING <td>Строка <td>string

    <tr><td>\ref LIBFPTR_UMO_COMMIT  <td>Сохранение изменений в постоянную память
        <td colspan="3">
        <td colspan="3">

   </table>

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_user_memory_operation(libfptr_handle handle);

/*!
   \ingroup libfptr_group_receipt

   \brief Допечатывает недопечатанный документ.

   \warning Если недопечатанных документов нет, не выдает ошибку.

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_continue_print(libfptr_handle handle);

/*!
   \ingroup libfptr_group_service

   \brief Инициализирует МГМ ФН.

   Пример использования:
   \code{.cpp}
    if (libfptr_init_mgm(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
*/
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_init_mgm(libfptr_handle handle);

/*!
   \ingroup libfptr_group_utils

   \brief Формирирует массив байтов с TLV-структурой, составленной из входящих реквизитов и их значений.

   Метод предназначен для формирования составных TLV-структур. Реквизиты могут повторяться.

   Пример использования (формирование значения реквизита 1223 (Данные агента)):
   \code{.cpp}
    libfptr_set_param_str(handle, 1005, L"ул. Свободы, 13");
    libfptr_set_param_str(handle, 1016, L"123456789047");
    libfptr_set_param_str(handle, 1026, L"Оператор 1");
    libfptr_set_param_str(handle, 1044, L"Оплата");
    libfptr_set_param_str(handle, 1073, L"+79121234567");
    libfptr_set_param_str(handle, 1074, L"+79121234568");
    libfptr_set_param_str(handle, 1074, L"+79121234574");
    libfptr_set_param_str(handle, 1075, L"+79121234569");
    if (libfptr_util_form_tlv(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    std::vector<uchar> agentData;
    int size = libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_TAG_VALUE, &agentData[0], agentData.size());
    if (size > agentData.size())
    {
        data.resize(size);
        libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_TAG_VALUE, &agentData[0], agentData.size());
    }

    // Передача сформированного тела реквизита в функцию регистрации
    libfptr_set_param_bytearray(handle, 1223, &agentData[0], agentData.size());
    // ...
    libfptr_registration(handle);
    // ...
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_util_form_tlv(libfptr_handle handle);

/*!
   \ingroup libfptr_group_utils

   \brief Формирует список доступных настроек

   <table>
    <caption>Общие параметры</caption>
    <tr><th>Название    <th>Описание    <th>Тип параметра <th>Обязательность или\nзначение по умолчанию
    <tr><th colspan="4">Входные параметры
    <tr><td>\ref LIBFPTR_PARAM_MAPPING_KEY      <td>Название настройки          <td>string <td>+
    <tr><th colspan="4">Выходные параметры
    <tr><td>\ref LIBFPTR_PARAM_MAPPING_VALUE    <td>Список значений настройки   <td>string <td>+
   </table>

   Список доступных для получения списка значений настроек (подробнее о настройках написано на странице \ref libfptr_page_settings):
   - \ref LIBFPTR_SETTING_MODEL - модель ККТ
   - \ref LIBFPTR_SETTING_PORT - способ связи с ККТ
   - \ref LIBFPTR_SETTING_COM_FILE - идентификатор COM / VCOM
   - \ref LIBFPTR_SETTING_USB_DEVICE_PATH - идентификатор USB
   - \ref LIBFPTR_SETTING_BAUDRATE - скорость канала COM-порта
   - \ref LIBFPTR_SETTING_BITS - битность канала COM-порта
   - \ref LIBFPTR_SETTING_STOPBITS - количество стоп-битов канала COM-порта
   - \ref LIBFPTR_SETTING_PARITY - четность канала COM-порта
   - \ref LIBFPTR_SETTING_EXISTED_COM_FILES - существующие COM-порты

   Формат \ref LIBFPTR_PARAM_MAPPING_VALUE - Json следующего вида:
   \code
    [
       {
          "key": <значение 1, строка>,
          "description": <описание значения 1, строка>
       },
       {
          "key": <значение 2, строка>,
          "description": <описание значения 2, строка>
       },
       {
          "key": <значение 3, строка>,
          "description": <описание значения 3, строка>
       }
    ]
   \endcode

   Пример использования:
   \code{.cpp}
    // Получение списка возможных значений настройки Port
    libfptr_set_param_string(handle, LIBFPTR_PARAM_MAPPING_KEY, LIBFPTR_SETTING_PORT);
    if (libfptr_util_mapping(handle) < 0)
    {
        // Обработка ошибки
        return;
    }
    std::vector<wchar_t> mapping(1024);
    int size = libfptr_get_param_str(handle, LIBFPTR_PARAM_MAPPING_VALUE, &mapping[0], mapping.size());
    if (size > mapping.length())
    {
        mapping.resize(size);
        libfptr_get_param_str(handle, LIBFPTR_PARAM_MAPPING_VALUE, &mapping[0], mapping.size());
    }
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_util_mapping(libfptr_handle handle);

/*!
   \ingroup libfptr_group_utils

   \brief Формирует код товарной номенклатуры (КТН) из заданных параметров

   Обязательным входным параметром для метода является \ref LIBFPTR_PARAM_NOMENCLATURE_TYPE
    (принимает значения из списка \ref libfptr_nomenclature_type),
    а также указанные ниже параметры в зависимости от типа КТН.

   <table>
    <caption>Список данных</caption>
    <tr><th rowspan="2">Тип КТН (LIBFPTR_PARAM_NOMENCLATURE_TYPE)    <th rowspan="2">Описание    <th colspan="3">Входные параметры
    <tr><th>Название<th>Описание<th>Тип параметра

    <tr><td rowspan="2">\ref LIBFPTR_NT_FURS                <td rowspan="2">Маркировка для изделий из меха
                            <td>\ref LIBFPTR_PARAM_GTIN                     <td>Идентификатор продукта GTIN <td>string
    <tr>        <td>\ref LIBFPTR_PARAM_SERIAL_NUMBER            <td>Серийный номер              <td>string

    <tr><td rowspan="2">\ref LIBFPTR_NT_MEDICINES           <td rowspan="2">Маркировка для лекарственных препаратов
                            <td>\ref LIBFPTR_PARAM_GTIN                     <td>Идентификатор продукта GTIN                 <td>string
    <tr>        <td>\ref LIBFPTR_PARAM_SERIAL_NUMBER            <td>Контрольный (идентификационный) знак (КиЗ)  <td>string

    <tr><td rowspan="2">\ref LIBFPTR_NT_TOBACCO             <td rowspan="2">Маркировка для табачной продукции
                            <td>\ref LIBFPTR_PARAM_GTIN                     <td>Идентификатор продукта GTIN                 <td>string
    <tr>        <td>\ref LIBFPTR_PARAM_SERIAL_NUMBER            <td>Код идентификации  <td>string
   </table>

   Пример использования:
   \code{.cpp}
    libfptr_set_param_str(handle, LIBFPTR_PARAM_GTIN, L"98765432101234");
    libfptr_set_param_str(handle, LIBFPTR_PARAM_SERIAL_NUMBER, L"ABC1234567890");
    if (libfptr_util_form_nomenclature(handle) < 0)
    {
        // Обработка ошибки
        return;
    }

    std::vector<uchar> nomenclatureCode;
    int size = libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_TAG_VALUE, &nomenclatureCode[0], nomenclatureCode.size());
    if (size > nomenclatureCode.size())
    {
        nomenclatureCode.resize(size);
        libfptr_get_param_bytearray(handle, LIBFPTR_PARAM_TAG_VALUE, &nomenclatureCode[0], nomenclatureCode.size());
    }

    // Передача сформированного тела реквизита в функцию регистрации позиции
    libfptr_set_param_bytearray(handle, 1162, &nomenclatureCode[0], nomenclatureCode.size());
    // ...
    libfptr_registration(handle);
    // ...
   \endcode

   \param handle Дескриптор
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_util_form_nomenclature(libfptr_handle handle);

/*!
   \ingroup libfptr_group_utils

   \brief Записывает сообщение в лог библиотеки

   Для корректной работы необходимо добавить к конфигурации логирования используемый тег (категорию).

   \param tag Тег (категория) сообщения
   \param level Уровень сообщения (\ref libfptr_log_level)
   \param message Сообщение
   \retval -1 Ошибка
   \retval 0 Успех
 */
DTOX_SHARED_EXPORT int DTOX_SHARED_CCA libfptr_log_write(const wchar_t *tag, int level,
                                                         const wchar_t *message);


#ifdef __cplusplus
}
#endif

#endif // ATOL_DTOX_FPTR_H
